#!/usr/bin/python
# -*- coding: UTF-8 -*-


class Dependencies(object):

    def __init__(self):
        try:
            import nibabel
            self.nibabel = nibabel
        except:
            self.nibabel = None
        try:
            import dicom
            self.dicom = dicom
        except:
            self.dicom = None
        try:
            import mne.io
            self.mne = mne
        except:
            self.mne = None
        try:
            import mnefun
            self.mnefun = mnefun
        except:
            self.mnefun = None


    def hasDependency(self, libname):
        if libname is None:
            return True # don't need dependency
        return getattr(self, libname)

    



#!/usr/bin/python
# -*- coding: UTF-8 -*-
import os
from niprov.filesystem import Filesystem
from niprov.commandline import Commandline
from niprov.filefilter import FileFilter
from niprov.adding import add


def discover(root, filefilter=FileFilter(), filesys=Filesystem(), 
        listener=Commandline()):
    """
    Search a directory for image files, and add them to your provenance collection.

    Files are only included if they match the filters in the 
    'discovery-filter.txt' file.
    Refer to niprov.add for details on what happens to individual files.

    Args:
        root (str): The top directory in which to look for new files.
    """
    dirs = filesys.walk(root)
    stats = {'total':0, 'new':0, 'series':0, 'failed':0, 'known':0}
    for (root, sdirs, files) in dirs:
        for filename in files:
            filepath = os.path.join(root, filename)
            if filefilter.include(filename):
                stats['total'] = stats['total'] + 1
                (p, status) = add(filepath, transient=False)
                stats[status] = stats[status] + 1
    listener.discoveryFinished(nnew=stats['new'], nadded=stats['series'], 
        nfailed=stats['failed'], ntotal=stats['total'])




#!/usr/bin/python
# -*- coding: UTF-8 -*-
from niprov.jsonfile import JsonFile
from niprov.filesystem import Filesystem
from niprov.commandline import Commandline
from niprov.files import FileFactory
from niprov.options import NiprovOptions
import errno
import copy


def log(new, transformation, parents, code=None, logtext=None, transient=False,
        script=None, provenance=None, opts=NiprovOptions(), 
        repository=JsonFile(), filesys=Filesystem(), listener=Commandline(), 
        factory=FileFactory()):
    """
    Register a transformation that creates a new image (or several).

    This will retrieve the primary parent's provenance. if no provenance is 
    availale for the primary parent, calls listener.unknownFile. Otherwise,
    some fields are copied from the primary parent, subject to availability. 
    For instance, if the parent has no 'subject' field, the new file's 
    provenance won't either.

    Args:
        new (str or list): Path(s) to the newly created file(s).
        transformation (str): Name of the operation that has been used.
        parents (str or list): Path(s) to the file(s) that were used as the 
            basis of the transformation. Assumes that the first file in the 
            list is the primary parent for which basic provenance is known.
        code (str, optional): Code that was used to generate the new file
        logtext (str, optional): Any information about the transformation that 
            was logged.
        script (str, optional): Path to the code file that contains the 
            transformation code.
        transient (bool, optional): Set this to True to indicate that the file 
            is only temporary and future checks should not expect it to be 
            physically present. Defaults to False, assuming that the file 
            remains.
        provenance (dict, optional): Add the key-value pairs in this dictionary 
            to the provenance record for the new files.
        opts (NiprovOptions): General settings for niprov. 
            See :py:mod:`niprov.options`

    Raises:
      IOError: '[Errno 2] File not found' is raised if the new file does not
        exist on the filesystem and is not marked as transient.

    Returns:
        dict or list: New provenance, if multiple files were created, this is 
            a list of dicts, otherwise, it is a single dict.
    """
    if isinstance(new, basestring):
        new = [new]
    if isinstance(parents, basestring):
        parents = [parents]
    if provenance is None:
        provenance = {}

    if opts.dryrun:
        transient = True

    #gather provenance common to all new files
    commonProvenance = provenance
    commonProvenance['parents'] = parents
    commonProvenance['transformation'] = transformation
    commonProvenance['script'] = script
    commonProvenance['transient'] = transient
    if code:
        commonProvenance['code'] = code
    if logtext:
        commonProvenance['logtext'] = logtext
    if repository.knowsByPath(parents[0]):
        parentProvenance = repository.byPath(parents[0]).provenance
        for field in ['acquired','subject','protocol']:
            if field in parentProvenance:
                commonProvenance[field] = parentProvenance[field]
    else:
        listener.unknownFile(parents[0])
        return

    #do things specific to each new file
    newImages = []
    for newfile in new:
        singleProvenance = copy.deepcopy(commonProvenance)
        singleProvenance['path'] = newfile
        img = factory.fromProvenance(singleProvenance)
        if not transient:
            if not filesys.fileExists(newfile):
                raise IOError(errno.ENOENT, 'File not found', newfile)
            img.inspect()
        if not opts.dryrun:
            repository.add(img.provenance)
        newImages.append(img)

    #only return one dict if only one new file was created
    if len(new) == 1:
        return img

    return newImages



    



import unittest
from mock import Mock, sentinel

class BaseExporterTests(unittest.TestCase):

    def test_If_export_called_with_list_calls_exportList(self):
        from niprov.exporter import BaseExporter
        exp = BaseExporter()
        exp.exportList = Mock()
        out = exp.export([sentinel.p1, sentinel.p2])
        exp.exportList.assert_called_with([sentinel.p1, sentinel.p2])
        self.assertEqual(out, exp.exportList())

    def test_If_export_called_with_list_calls_exportSingle(self):
        from niprov.exporter import BaseExporter
        exp = BaseExporter()
        exp.exportSingle = Mock()
        out = exp.export(sentinel.p1)
        exp.exportSingle.assert_called_with(sentinel.p1)
        self.assertEqual(out, exp.exportSingle())

    def test_If_created_with_narrative_calls_exportNarrative(self):
        from niprov.exporter import BaseExporter
        exp = BaseExporter(form='narrative')
        exp.exportNarrative = Mock()
        out = exp.export(sentinel.p1)
        exp.exportNarrative.assert_called_with(sentinel.p1)
        self.assertEqual(out, exp.exportNarrative())






import os
import datetime


class Filesystem(object):
    """Wrapper of filesystem access functionality such as that implemented by 
    the os package in the standard library.
    """

    def __init__(self):
        self.open = open

    def fileExists(self, path):
        return os.path.isfile(path) 

    def walk(self, path):
        return os.walk(path)

    def readlines(self, path):
        with open(path) as fhandle:
            lines = fhandle.read().splitlines()
        return lines

    def read(self, path):
        """Read the contents of a textfile.

        Args:
            path: Path to the file to read.

        Returns:
            str: Contents of the file

        Raises:
            IOError: [Errno 2] No such file or directory: 'xyz'
        """
        with open(path) as fhandle:
            contents = fhandle.read()
        return contents

    def write(self, path, content):
        """Write string content to a textfile.

        Args:
            path: Path to the file to read.
            content (str): What to fill the file with
        """
        with open(path, 'w') as fhandle:
            fhandle.write(content)

    def getsize(self, path):
        return os.path.getsize(path)

    def getctime(self, path):
        """Get the creation time for the file at path.

        Args:
            path: Path to the file to read.

        Returns:
            datetime: Time when the file was last changed
        """
        return datetime.datetime.fromtimestamp(os.path.getctime(path))




#!/usr/bin/python
# -*- coding: UTF-8 -*-
from niprov.jsonfile import JsonFile
from niprov.exporters import ExportFactory
from niprov.commandline import Commandline


def report(medium=None, form=None, forFile=None, forSubject=None, 
        repository=JsonFile(), exportFactory=ExportFactory(), 
        listener=Commandline()):
    """Publish or simply return provenance for selected files.

    To get provenance on one specific file, pass its path as the 'forFile' 
    argument. Alternatively, to get all files associated with a certain subject,
    use the 'forSubject' argument. If none of these is used, provenance for all
    files is reported.

    Args:
        medium (str): The medium in which to publish the provenance. 
            One of 'html' (save as html and open in firefox),
                'stdout' (print the provenance to the terminal)
        forFile (str): Select one file based on this path.
        forSubject (str): Select files regarding this subject.

    Returns:
        Provenance reported. Either a list of dicts, or a dict.
    """
    exporter = exportFactory.createExporter(medium, form)
    if forFile:
        if not repository.knowsByPath(forFile):
            listener.unknownFile(forFile)
            return
        provenance = repository.byPath(forFile)
    elif forSubject:
        provenance = repository.bySubject(forSubject)
    else:
        provenance = repository.all()
    return exporter.export(provenance)




#!/usr/bin/python
# -*- coding: UTF-8 -*-
import pkg_resources
from niprov.filesystem import Filesystem


class FileFilter(object):

    def __init__(self, filesys=Filesystem()):
        filterfile = pkg_resources.resource_filename(
            'niprov','discovery-filter.txt')
        self.filters = filesys.readlines(filterfile)

    def include(self, filepath):
        """Whether the file is to be included in discovery.

        Args:
            filepath (str): The full path of the file.

        Returns:
            bool: True if the file should be included.

        Raises:
            ValueError: If not able to read discovery-filter.txt
        """
#        if not self.filters:
#            raise ValueError('Was not able to load filter file.')
        for filt in self.filters:
            if filt in filepath:
                return True
        return False



from __future__ import print_function
import unittest
from mock import Mock



class RecordingTests(unittest.TestCase):

    def setUp(self):
        import niprov.recording as recording
        self.log = Mock()
        self.opts = Mock()
        self.opts.dryrun = False
        self.sub = Mock()
        self.listener = Mock()
        recording.log = self.log
        self.recording = recording

    def record(self, cmd, **kwargs):
        self.recording.record(cmd, externals=self.sub, listener=self.listener, 
            opts=self.opts, **kwargs)

    def test_Executes_commands(self):
        cmd = ['mytransform','-out','newfile.f','-in','oldfile.f']
        self.record(cmd)
        self.sub.run.assert_any_call(cmd)

    def test_Extracts_provenance_from_commands(self):
        cmd = ['mytransform','-out','newfile.f','-in','oldfile.f']
        self.record(cmd)
        self.log.assert_called_with(['newfile.f'],'mytransform',['oldfile.f'], 
            transient=False, code=' '.join(cmd), logtext=self.sub.run().output, 
            script=None, opts=self.opts, provenance={})

    def test_If_parent_or_new_provided_override_parsed(self):
        cmd = ['mytransform','-out','newfile.f','-in','oldfile.f']
        self.record(cmd, parents=['customParent'], new='customNew')
        self.log.assert_called_with(['customNew'],'mytransform',['customParent'],
            transient=False, code=' '.join(cmd), logtext=self.sub.run().output, 
            script=None, opts=self.opts, provenance={})

    def test_Passes_transient_flag(self):
        cmd = ['mytransform','-out','newfile.f','-in','oldfile.f']
        self.record(cmd, transient=True)
        self.log.assert_called_with(['newfile.f'],'mytransform',['oldfile.f'], 
            transient=True, code=' '.join(cmd), logtext=self.sub.run().output, 
            script=None, opts=self.opts, provenance={})

    def test_Informs_listener_about_interpretation(self):
        cmd = ['mytransform','-out','newfile.f','-in','oldfile.f']
        self.record(cmd)
        self.listener.interpretedRecording.assert_called_with(
            ['newfile.f'],'mytransform',['oldfile.f'])

    def test_Works_on_single_string_too(self):
        cmd = 'mytransform -out newfile.f -in oldfile.f'
        self.record(cmd)
        self.sub.run.assert_called_with(cmd.split())
        self.log.assert_called_with(['newfile.f'],'mytransform',['oldfile.f'], 
            transient=False, code=cmd, logtext=self.sub.run().output, 
            script=None, opts=self.opts, provenance={})

    def test_Python_code(self):
        myfunc = Mock()
        myfunc.side_effect = lambda a,b,one=None,two=None: None
        myfunc.func_name = 'myfunc'
        args = ['foo','bar']
        kwargs = {'one':'foz','two':'baz'}
        self.record(myfunc, args=args, kwargs=kwargs, new='new.f', parents=['old.f'])
        myfunc.assert_called_with(*args, **kwargs)
        self.log.assert_called_with(['new.f'],myfunc.func_name,['old.f'], 
            transient=False, code=None, logtext='', 
            script=myfunc.func_code.co_filename,
            opts=self.opts, provenance={'args':args, 'kwargs':kwargs})

    def test_Python_code_output_captured(self):
        def myfunc():
            print('Hello MyFunc')
        args = []
        kwargs = {}
        self.record(myfunc, args=args, kwargs=kwargs, new='new.f', parents=['old.f'])
        self.log.assert_called_with(['new.f'],myfunc.func_name,['old.f'], 
            transient=False, code=None, logtext='Hello MyFunc\n', 
            script=myfunc.func_code.co_filename,
            opts=self.opts, provenance={'args':args, 'kwargs':kwargs})

    def test_On_dryrun_doesnt_execute_python_code(self):
        myfunc = Mock()
        myfunc.side_effect = lambda a,b,one=None,two=None: None
        myfunc.func_name = 'myfunc'
        args = ['foo','bar']
        kwargs = {'one':'foz','two':'baz'}
        self.opts.dryrun = True
        self.record(myfunc, args=args, kwargs=kwargs)
        assert not myfunc.called, "Passed function was called on dry run."

    def test_On_dryrun_doesnt_execute_commands(self):
        self.opts.dryrun = True
        cmd = ['mytransform','-out','newfile.f','-in','oldfile.f']
        self.record(cmd)
        assert not self.sub.run.called, "Command was executed on dry run."

    def test_Passes_opts_to_log(self):
        cmd = ['mytransform','-out','newfile.f','-in','oldfile.f']
        self.record(cmd, transient=True)
        self.log.assert_called_with(['newfile.f'],'mytransform',['oldfile.f'], 
            transient=True, code=' '.join(cmd), logtext=self.sub.run().output, 
            script=None, provenance={}, opts=self.opts)

    def test_Abbreviated_versions_of_out_and_in(self):
        cmd = ['mytransform','-o','newfile.f','-i','oldfile.f']
        self.record(cmd)
        self.log.assert_called_with(['newfile.f'],'mytransform',['oldfile.f'], 
            transient=False, code=' '.join(cmd), logtext=self.sub.run().output, 
            script=None, opts=self.opts, provenance={})

    def test_Gives_listener_options(self):
        self.record(['mytransform','-out','newfile.f','-in','oldfile.f'])
        self.listener.setOptions.assert_called_with(self.opts)

    def test_Passes_listener_bash_command(self):
        def myfunc():
            print('Hello MyFunc')
        args = []
        kwargs = {}
        self.record(myfunc, args=args, kwargs=kwargs, new='new.f', parents=['old.f'])
        assert not self.listener.receivedBashCommand.called
        cmd = ['mytransform','-o','newfile.f','-i','oldfile.f']
        self.record(cmd)
        self.listener.receivedBashCommand.assert_called_with(cmd)




import os, shutil, glob
from niprov.commandline import Commandline


def renameDicoms(dicomdir, listener=Commandline()):
    """ Add the .dcm extension to any non-hidden files without extension.

        Args:
            dicomdir (str): Directory in which to rename files.
    """
    for f in glob.glob(os.path.join(dicomdir, '*')):
        if '.' not in os.path.basename(f):
            listener.renamedDicom(f)
            shutil.move(f, f+'.dcm')



import unittest
from mock import Mock, patch


class MnefunTests(unittest.TestCase):

    def setUp(self):
        self.listener = Mock()
        self.libs = Mock()

    def test_Discovers_fetched_raw_files(self):
        import niprov.mnefunsupport
        class MockParams(object):
            pass
        params = MockParams()
        params.work_dir = '/root'
        params.subjects = ['johndoe','janedoe']
        params.raw_dir = 'rawdir'
        def fetch_raw_files():
            pass
        with patch('niprov.mnefunsupport.discover') as discover:
            niprov.mnefunsupport.handler('Pulling raw files from acquisition machine', 
                fetch_raw_files, None, params, listener=self.listener)
            discover.assert_any_call('/root/johndoe/rawdir')
            discover.assert_any_call('/root/janedoe/rawdir')
            self.listener.mnefunEventReceived.assert_called_with('fetch_raw_files')

    def test_Logs_sss_operation(self):
        import niprov.mnefunsupport
        class MockParams(object):
            pass
        params = MockParams()
        params.subjects = ['s1','s2']
        fnames = {'raw':{'s1':['subj 1 raw file 1','subj 1 raw file 2'],
                        's2':['subj 2 raw file 1','subj 2 raw file 2']},
                'sss': {'s1':['subj 1 sss file 1','subj 1 sss file 2'],
                        's2':['subj 2 sss file 1','subj 2 sss file 2']}}
        self.libs.mnefun.get_raw_fnames.side_effect = lambda p, s, t: fnames[t][s]
        def fetch_sss_files():
            pass
        with patch('niprov.mnefunsupport.log') as log:
            niprov.mnefunsupport.handler('Pulling SSS files from remote workstation', 
                fetch_sss_files, None, params, listener=self.listener, libs=self.libs)
            log.assert_any_call(fnames['sss']['s1'][0],
                                'Signal Space Separation',
                                fnames['raw']['s1'][0], provenance = {'mnefun':{}})
            log.assert_any_call(fnames['sss']['s2'][1],
                                'Signal Space Separation',
                                fnames['raw']['s2'][1], provenance = {'mnefun':{}})

    def test_Logs_ssp_operation(self):
        import niprov.mnefunsupport
        class MockParams(object):
            pass
        params = MockParams()
        params.subjects = ['s1','s2']
        fnames = {'pca':{'s1':['subj 1 pca file 1','subj 1 pca file 2'],
                        's2':['subj 2 pca file 1','subj 2 pca file 2']},
                'sss': {'s1':['subj 1 sss file 1','subj 1 sss file 2'],
                        's2':['subj 2 sss file 1','subj 2 sss file 2']}}
        self.libs.mnefun.get_raw_fnames.side_effect = lambda p, s, t: fnames[t][s]
        def apply_preprocessing_combined():
            pass
        with patch('niprov.mnefunsupport.log') as log:
            niprov.mnefunsupport.handler('Apply SSP vectors and filtering.', 
                apply_preprocessing_combined, None, params, 
                listener=self.listener, libs=self.libs)
            log.assert_any_call(fnames['pca']['s1'][0],
                                'Signal Space Projection',
                                fnames['sss']['s1'][0], provenance = {'mnefun':{}})
            log.assert_any_call(fnames['pca']['s2'][1],
                                'Signal Space Projection',
                                fnames['sss']['s2'][1], provenance = {'mnefun':{}})

    def test_Logs_epoch_operation(self):
        import niprov.mnefunsupport
        class MockParams(object):
            pass
        params = MockParams()
        params.subjects = ['s1','s2']
        params.analyses = ['a']
        fnames = {'pca':{'s1':['subj 1 pca file 1','subj 1 pca file 2'],
                        's2':['subj 2 pca file 1','subj 2 pca file 2']}}
        epochfnames = {'s1':['s1 evt 1','s1 evt 2','s1 evt 3'],
                        's2':['s2 evt 1','s2 evt 2','s2 evt 3']}
        self.libs.mnefun.get_raw_fnames.side_effect = \
            lambda p, s, t: fnames[t][s]
        self.libs.mnefun._paths.get_epochs_evokeds_fnames.side_effect = \
            lambda p, s, a: epochfnames[s]
        def save_epochs():
            pass
        with patch('niprov.mnefunsupport.log') as log:
            niprov.mnefunsupport.handler('Doing epoch EQ/DQ', 
                save_epochs, None, params, listener=self.listener, libs=self.libs)
            log.assert_any_call(epochfnames['s1'][0], #any event file
                                'Epoching',
                                fnames['pca']['s1'], provenance = {'mnefun':{}}) #all raw files for subj
            log.assert_any_call(epochfnames['s2'][1], #any event file
                                'Epoching',
                                fnames['pca']['s2'], provenance = {'mnefun':{}}) #all raw files for subj

    def test_Log_seeded_with_params_based_custom_provenance(self):
        import niprov.mnefunsupport
        class MockParams(object):
            pass
        params = MockParams()
        params.tmin = -0.2
        params.quat_tol = 5e-2
        params.subjects = ['s1','s2']
        fnames = {'raw':{'s1':['subj 1 raw file 1','subj 1 raw file 2'],
                        's2':['subj 2 raw file 1','subj 2 raw file 2']},
                'sss': {'s1':['subj 1 sss file 1','subj 1 sss file 2'],
                        's2':['subj 2 sss file 1','subj 2 sss file 2']}}
        self.libs.mnefun.get_raw_fnames.side_effect = lambda p, s, t: fnames[t][s]
        def fetch_sss_files():
            pass
        with patch('niprov.mnefunsupport.log') as log:
            niprov.mnefunsupport.handler('Pulling SSS files from remote workstation', 
                fetch_sss_files, None, params, listener=self.listener, libs=self.libs)
            log.assert_called_with(fnames['sss']['s2'][1],
                                'Signal Space Separation',
                                fnames['raw']['s2'][1], 
                                provenance={'mnefun':{
                                    'tmin':-0.2,
                                    'quat_tol':5e-2}})


#    write_epochs : bool
#        Write epochs to disk.
#    gen_covs : bool
#        Generate covariances.
#    gen_fwd : bool
#        Generate forward solutions.
#    get_inv : bool
#        Generate inverses.
#    gen_report : bool
#        Generate HTML reports.




from datetime import datetime
from niprov.basefile import BaseFile
from niprov.dependencies import Dependencies


class FifFile(BaseFile):

    def __init__(self, fpath, dependencies=Dependencies(), **kwargs):
        super(FifFile, self).__init__(fpath, **kwargs)
        self.libs = dependencies

    def inspect(self):
        provenance = super(FifFile, self).inspect()
        img = self.libs.mne.io.Raw(self.path, allow_maxshield=True)
        sub = img.info['subject_info']
        if sub is not None:
            provenance['subject'] = sub['first_name']+' '+sub['last_name']
        provenance['project'] = img.info['proj_name']
        acqTS = img.info['meas_date'][0]
        provenance['acquired'] = datetime.fromtimestamp(acqTS)
        T = img.last_samp - img.first_samp + 1
        provenance['dimensions'] = [img.info['nchan'], T]
        return provenance

    def attach(self):
        """
        Attach the current provenance to the file by appending it as a
        json-encoded string to the 'description' header field.
        """
        info = self.libs.mne.io.read_info(self.path)
        provstr = self.serializer.serialize(self.provenance)
        info['description'] = info['description']+' NIPROV:'+provstr
        self.libs.mne.io.write_info(self.path, info)




#!/usr/bin/python
# -*- coding: UTF-8 -*-
import unittest
from mock import Mock
from datetime import datetime
import json
import random
import copy


class StorageTests(unittest.TestCase):

    def setUp(self):
        r = random.randint(1000,9999)
        self.templocation = '/var/tmp/provenance-{0}.json'.format(r)
        self.fileFactory = Mock()

    def test_Can_add_file_and_retrieve_it(self):
        from niprov.jsonfile import JsonFile
        repo = JsonFile(factory=self.fileFactory)
        repo.datafile = self.templocation
        provenance = self.sampleProvenanceRecord()
        repo.add(provenance)
        repo2 = JsonFile(factory=self.fileFactory)
        repo2.datafile = self.templocation
        out = repo2.byPath(provenance['path'])
        self.fileFactory.fromProvenance.assert_called_with(provenance)
        self.assertEqual(out, self.fileFactory.fromProvenance())

    def test_If_file_doesnt_exist_all_returns_empty_list(self):
        from niprov.jsonfile import JsonFile
        filesys = Mock()
        filesys.read.side_effect = IOError(
            "[Errno 2] No such file or directory: 'provenance.json'")
        repo = JsonFile(filesys)
        repo.datafile = self.templocation
        out = repo.all()
        self.assertEqual(out, [])

    def test_If_no_provenance_known_for_file_byPath_raises_IndexError(self):
        from niprov.jsonfile import JsonFile
        serializer = Mock()
        serializer.deserializeList.return_value = []
        repo = JsonFile(json=serializer)
        repo.datafile = self.templocation
        with self.assertRaises(IndexError):
            repo.byPath('nothere')

    def test_Can_add_file_and_ask_whether_its_known(self):
        from niprov.jsonfile import JsonFile
        repo = JsonFile(factory=self.fileFactory)
        repo.datafile = self.templocation
        provenance = self.sampleProvenanceRecord()
        repo.add(provenance)
        repo2 = JsonFile(factory=self.fileFactory)
        repo2.datafile = self.templocation
        self.assertFalse(repo2.knowsByPath('nonexisting'))
        self.assertTrue(repo2.knowsByPath(provenance['path']))

    def test_Can_add_file_and_ask_whether_its_known(self):
        from niprov.jsonfile import JsonFile
        repo = JsonFile(factory=self.fileFactory)
        repo.datafile = self.templocation
        provenance = self.sampleProvenanceRecord()
        repo.add(provenance)
        repo2 = JsonFile(factory=self.fileFactory)
        repo2.datafile = self.templocation
        known = Mock()
        known.path = provenance['path']
        unknown = Mock()
        unknown.path = 'maohafnv'
        self.assertFalse(repo2.knows(unknown))
        self.assertTrue(repo2.knows(known))

    def test_Can_add_file_and_ask_whether_its_series_is_known(self):
        from niprov.jsonfile import JsonFile
        repo = JsonFile(factory=self.fileFactory)
        repo.datafile = self.templocation
        provenance = self.sampleProvenanceRecord()
        SUID = 'iamtheseries'
        provenance['seriesuid'] = SUID
        repo.add(provenance)
        repo2 = JsonFile(factory=self.fileFactory)
        repo2.datafile = self.templocation
        knownSeries = Mock()
        knownSeries.path = 'apath1'
        knownSeries.getSeriesId.return_value = SUID
        unknown = Mock()
        unknown.path = 'anotherpath'
        self.assertFalse(repo2.knows(knownSeries)) #file itself not know
        self.assertFalse(repo2.knowsSeries(unknown)) #other series not known
        self.assertTrue(repo2.knowsSeries(knownSeries)) #but this file's series known

    def test_Can_add_file_and_ask_whether_its_series_is_known(self):
        from niprov.jsonfile import JsonFile
        repo = JsonFile(factory=self.fileFactory)
        repo.datafile = self.templocation
        provenance = self.sampleProvenanceRecord()
        provenance['seriesuid'] = None
        repo.add(provenance)
        noneSeries = Mock()
        noneSeries.getSeriesId.return_value = None
        self.assertFalse(repo.knowsSeries(noneSeries))

    def test_Can_get_provenance_for_series_with_getSeries(self):
        from niprov.jsonfile import JsonFile
        repo = JsonFile(factory=self.fileFactory)
        repo.datafile = self.templocation
        prov1 = self.sampleProvenanceRecord()
        prov2 = self.sampleProvenanceRecord()
        prov2['seriesuid'] = 's8m3d0s2'
        repo.add(prov1)
        repo.add(prov2)
        img = Mock()
        img.getSeriesId.return_value = prov2['seriesuid']
        img2 = Mock()
        img2.getSeriesId.return_value = 'a9m2l4a8'
        out = repo.getSeries(img)
        self.fileFactory.fromProvenance.assert_called_with(prov2)
        self.assertEqual(out, self.fileFactory.fromProvenance())
        with self.assertRaises(IndexError):
            repo.getSeries(img2)

    def test_Update(self):
        from niprov.jsonfile import JsonFile
        repo = JsonFile(factory=self.fileFactory)
        repo.datafile = self.templocation
        provenance = self.sampleProvenanceRecord()
        repo.add(provenance)
        intermediate = copy.deepcopy(provenance)
        intermediate['newfield'] = 'newval'
        intermediate['subject'] = 'Jane Newman'
        repo3 = JsonFile(factory=self.fileFactory)
        repo3.datafile = self.templocation
        updatedImage = Mock()
        updatedImage.path = intermediate['path']
        updatedImage.provenance = intermediate
        repo3.update(updatedImage)
        repo4 = JsonFile(factory=self.fileFactory)
        repo4.datafile = self.templocation
        out = repo4.byPath(provenance['path'])
        self.fileFactory.fromProvenance.assert_called_with( intermediate)

    def test_ByPath_also_returns_series_if_filepath_among_it(self):
        from niprov.jsonfile import JsonFile
        repo = JsonFile(factory=self.fileFactory)
        repo.datafile = self.templocation
        provenance = self.sampleProvenanceRecord()
        provenance['filesInSeries'] = ['sfile1.f','sfile2.f']
        repo.add(provenance)
        repo2 = JsonFile(factory=self.fileFactory)
        repo2.datafile = self.templocation
        out = repo2.byPath('sfile1.f')
        self.fileFactory.fromProvenance.assert_called_with(provenance)
        with self.assertRaises(IndexError):
            repo2.byPath('sfile3.f')

#    def test_byApproval_calls_all_and_filters_based_on_approval(self):
#        from niprov.jsonfile import JsonFile
#        repo = JsonFile(factory=self.fileFactory)
#        img1 = self.imgWithApproval('xyz')
#        img2 = self.imgWithoutApproval()
#        img3 = self.imgWithApproval('xyz')
#        img4 = self.imgWithApproval('abc')
#        repo.all = Mock()
#        repo.all.return_value = [img1, img2, img3, img4]
#        returned = repo.byApproval('xyz')
#        self.assertEqual(returned, [img1, img3])

    def imgWithApproval(self, approvalStatus):
        img = Mock()
        img.provenance = {}
        img.provenance['approval'] = approvalStatus
        return img

    def imgWithoutApproval(self):
        img = Mock()
        img.provenance = {}
        return img

    def sampleProvenanceRecord(self):
        record = {}
        record['path'] = str(random.randint(1000,9999))
        record['subject'] = 'John Doeish'
        record['protocol'] = 'T1 SENSE'
        record['acquired'] = datetime(2014, 8, 5, 12, 19, 14)
        return record






import sys 
from cStringIO import StringIO 


class OutputCapture(object):
    """Context manager which records stdout writes.

    Used when recording python-based transformations.
    """

    def __enter__(self):
        self.oldstdout = sys.stdout 
        sys.stdout = self.stdout = StringIO()
        return self

    def __exit__(self, type, value, traceback):
        sys.stdout = self.oldstdout
        self.output = self.stdout.getvalue()
        self.stdout.close()



import unittest
from mock import Mock
from datetime import datetime

class BaseFileTests(unittest.TestCase):

    def setUp(self):
        self.log = Mock()
        self.hasher = Mock()
        self.filesys = Mock()
        self.json = Mock()
        self.path = 'example.abc'
        from niprov.basefile import BaseFile
        self.constructor = BaseFile
        self.ckwargs = {}
        self.file = BaseFile(self.path, listener=self.log, 
            filesystem=self.filesys, hasher=self.hasher, serializer=self.json)

    def test_Saves_file_path_along_with_provenance(self):
        out = self.file.inspect()
        self.assertEqual(out['path'], self.path)

    def test_Saves_filesize_along_with_provenance(self):
        out = self.file.inspect()
        self.assertEqual(out['size'], self.filesys.getsize(self.path))

    def test_Saves_file_creation_time_along_with_provenance(self):
        out = self.file.inspect()
        self.assertEqual(out['created'], self.filesys.getctime(self.path))

    def test_Asks_hasher_for_digest_of_file(self):
        out = self.file.inspect()
        self.assertEqual(out['hash'], self.hasher.digest(self.path))

    def test_Provenance_property_equals_dictionary_returned_by_inspect(self):
        out = self.file.inspect()
        self.assertEqual(out, self.file.provenance)

    def test_Attach_method(self):
        self.file.provenance = {'foo':'bar'}
        self.file.attach()
        self.json.serialize.assert_called_with(self.file.provenance)
        self.filesys.write.assert_called_with(
            self.path+'.provenance', self.json.serialize(self.file.provenance))

    def test_Series_interface(self):
        self.assertEqual(self.file.getSeriesId(), None)

    def test_Construct_with_provenance(self):
        prov = {'aprop':'aval'}
        img = self.constructor(self.path, provenance=prov)
        self.assertEqual(prov, img.provenance)

    def test_Construct_with_provenance_adds_path_if_not_set(self):
        prov = {'aprop':'aval'}
        img = self.constructor(self.path, provenance=prov)
        self.assertIn('path', img.provenance)
        self.assertEqual(self.path, img.provenance['path'])

    def test_Inspect_leaves_existing_fields_updates_others(self):
        prov = {'aprop':'aval','size':9876}
        img = self.constructor(self.path, provenance=prov, 
            filesystem=self.filesys, hasher=self.hasher, **self.ckwargs)
        img.inspect()
        self.assertEqual(img.provenance['aprop'], 'aval')
        self.assertEqual(img.provenance['size'], self.filesys.getsize(self.path))


#ATTACH
# check if we have provenance to attach
# call method for type-specific attachment behavior (file or insert)



#READATTACHED
# call method for type-specific attachment behavior (file or insert)
# return provenance
# save provenance to instance



import unittest
from mock import Mock

class ExportFactoryTests(unittest.TestCase):

    def test_Html(self):
        from niprov.exporters import ExportFactory
        from niprov.html import HtmlExporter
        exporter = ExportFactory().createExporter('html', None)
        self.assertIsInstance(exporter, HtmlExporter)

    def test_Default(self):
        from niprov.exporters import ExportFactory
        from niprov.directexporter import DirectExporter
        exporter = ExportFactory().createExporter(None, None)
        self.assertIsInstance(exporter, DirectExporter)

    def test_StdOut(self):
        from niprov.exporters import ExportFactory
        from niprov.stdout import StandardOutputExporter
        exporter = ExportFactory().createExporter('stdout', None)
        self.assertIsInstance(exporter, StandardOutputExporter)

    def test_Form_supplied(self):
        from niprov.exporters import ExportFactory
        exporter = ExportFactory().createExporter(None, 'narrative')
        self.assertEqual(exporter.form, 'narrative')
        exporter = ExportFactory().createExporter('html', None)
        self.assertEqual(exporter.form, None)
        exporter = ExportFactory().createExporter('stdout', 'narrative')
        self.assertEqual(exporter.form, 'narrative')







import unittest
from mock import Mock, MagicMock, PropertyMock
from datetime import datetime
from tests.test_basefile import BaseFileTests


class DicomTests(BaseFileTests):

    def setUp(self):
        super(DicomTests, self).setUp()
        self.libs = Mock()
        self.setupPydicom()
        from niprov.dcm import DicomFile
        self.constructor = DicomFile
        self.ckwargs = {'dependencies':self.libs}
        self.file = DicomFile(self.path, listener=self.log, 
            filesystem=self.filesys, hasher=self.hasher, dependencies=self.libs,
            serializer=self.json)

    def test_Gets_basic_info_from_pydicom_and_returns_it(self):
        out = self.file.inspect()
        self.assertEqual(out['subject'], 'John Doeish')
        self.assertEqual(out['protocol'], 'T1 SENSE')
        self.assertEqual(out['acquired'], datetime(2014, 8, 5, 12, 19, 14))

    def test_if_doesnt_have_acqDateTime_get_seriesDatetime(self):
        del(self.img.AcquisitionDateTime)
        self.img.SeriesDate = '20120416'
        self.img.SeriesTime = '101330.60000'
        h = datetime.fromtimestamp(101330.60000).hour
        out = self.file.inspect()
        self.assertEqual(out['acquired'], 
            datetime(2012, 4, 16, h, 8, 50))

    def test_Series_interface(self):
        self.assertEqual(self.file.getSeriesId(), self.img.SeriesInstanceUID)
        self.assertIn(self.file.path, self.file.provenance['filesInSeries'])
        newFile = Mock()
        self.file.addFile(newFile)
        self.assertIn(newFile.path, self.file.provenance['filesInSeries'])

    def test_Gets_dimensions(self):
        out = self.file.inspect()
        self.assertEqual(out['dimensions'], [11, 12, 13])
        del(self.img.NumberOfFrames)
        out = self.file.inspect()
        self.assertEqual(out['dimensions'], [11, 12, 1])
        self.file.addFile(Mock())
        self.assertEqual(out['dimensions'], [11, 12, 2])

    def test_File_without_Rows(self):
        del(self.img.Rows)
        out = self.file.inspect()
        assert not self.log.fileError.called
        self.assertNotIn('dimensions', out)
        del(self.img.NumberOfFrames)
        out = self.file.inspect()
        self.file.addFile(Mock())
        assert not self.log.fileError.called

    def setupPydicom(self):
        self.img = Mock()
        self.img.AcquisitionDateTime = '20140805121914.59000'
        self.img.SeriesDescription = 'T1 SENSE'
        self.img.PatientID = 'John Doeish'
        self.img.SeriesInstanceUID = '1.3.46.670589.11.17388.5.0.6340.2011121308140690488'
        self.img.Rows = 11
        self.img.Columns = 12
        self.img.NumberOfFrames = 13
        self.libs.dicom.read_file.return_value = self.img
        self.libs.hasDependency.return_value = True




from datetime import datetime
from niprov.basefile import BaseFile
from niprov.dependencies import Dependencies


class NiftiFile(BaseFile):

    def __init__(self, fpath, dependencies=Dependencies(), **kwargs):
        super(NiftiFile, self).__init__(fpath, **kwargs)
        self.libs = dependencies

    def attach(self):
        """
        Attach the current provenance to the file by injecting it as a 
        json-encoded extension to the nifti header.
        """
        img = self.libs.nibabel.load(self.path)
        provstr = self.serializer.serialize(self.provenance)
        ext = self.libs.nibabel.nifti1.Nifti1Extension('comment', provstr)
        hdr = img.get_header()
        hdr.extensions.append(ext)
        img.to_filename(self.path)



#!/usr/bin/python
# -*- coding: UTF-8 -*-
from niprov.jsonfile import JsonFile
from niprov.commandline import Commandline


def markForApproval(files, repository=JsonFile()):
    """Mark a list of files for approval by a human.

    Args:
        files (list): List of paths of files tracked by niprov to mark for
            approval.
    """
    for filepath in files:
        repository.updateApproval(filepath,'pending')

def markedForApproval(repository=JsonFile(), listener=Commandline()):
    """List files marked for approval by a human.
    """
    markedFiles = repository.byApproval('pending')
    listener.filesMarkedForApproval(markedFiles)
    return markedFiles

def approve(filepath, repository=JsonFile()):
    """Mark this file as approved.

    Args:
        filepath (str): Path to the tracked file that has been found valid.
    """
    repository.updateApproval(filepath,'granted')

def selectApproved(files, repository=JsonFile()):
    """Return only files that have approval status 'granted'.

    Args:
        files (list): List of paths of files to check for approval status.
    """
    selection = []
    for filepath in files:
        img = repository.byPath(filepath)
        if 'approval' in img.provenance:
            if img.provenance['approval'] == 'granted':
                selection.append(img.path)
    return selection



# -*- coding: utf-8 -*-
"""mnefun support module

This module provides handlers to attach to mnefun events.

::

    import mnefun
    import niprov.mnefunsupport
    params = mnefun.Params()
    params.on_process = niprov.mnefunsupport.handler

"""
import os
from niprov.commandline import Commandline
from niprov.dependencies import Dependencies
from niprov.discovery import discover
from niprov.logging import log


def handler(text, func, out, params, listener=Commandline(), 
    libs=Dependencies()):
    """mnefun on_process handler

    Responds to the following mnefun steps:
    fetch_raw_files: Runs discover on the subjects' raw data dirs.
    fetch_sss_files,
    apply_preprocessing_combined,
    save_epochs:
        For these steps, runs log for the new files.
    """
    funcname = func.func_name
    listener.mnefunEventReceived(funcname)
    paramdict = {}
    for param in provenanceParams:
        if hasattr(params, param):
            paramdict[param] = getattr(params, param)
    customprov = {'mnefun':paramdict}
    for subj in params.subjects:
        if funcname == 'fetch_raw_files':
            subjrawdir = os.path.join(params.work_dir, subj, params.raw_dir)
            discover(subjrawdir)
        elif funcname == 'fetch_sss_files':
            trans = 'Signal Space Separation'
            rawfiles = libs.mnefun.get_raw_fnames(params, subj, 'raw')
            sssfiles = libs.mnefun.get_raw_fnames(params, subj, 'sss')
            for rawfile, sssfile in zip(rawfiles, sssfiles):
                log(sssfile, trans, rawfile, provenance=customprov)
        elif funcname == 'apply_preprocessing_combined':
            trans = 'Signal Space Projection'
            sssfiles = libs.mnefun.get_raw_fnames(params, subj, 'sss')
            pcafiles = libs.mnefun.get_raw_fnames(params, subj, 'pca')
            for sssfile, pcafile in zip(sssfiles, pcafiles):
                log(pcafile, trans, sssfile, provenance=customprov)
        elif funcname == 'save_epochs':
            pcafiles = libs.mnefun.get_raw_fnames(params, subj, 'pca')
            evtfiles = libs.mnefun._paths.get_epochs_evokeds_fnames(params, 
                subj, params.analyses)
            for evtfile in evtfiles:
                log(evtfile, 'Epoching', pcafiles, provenance=customprov)


provenanceParams = [
'tmin',
'tmax',
't_adjust',
'bmin',
'bmax',
'filter_length',
'cont_lp',
'lp_cut',
'proj_sfreq',
'decim',
'drop_thresh',
'bem_type',
'fwd_mindist',
'auto_bad',
'auto_bad_reject',
'auto_bad_flat',
'auto_bad_meg_thresh',
'auto_bad_eeg_thresh',
'ecg_channel',
'eog_channel',
'translate_positions',
'quat_tol',
'tsss_dur',
'mri']


#        self.reject = dict(eog=np.inf, grad=1500e-13, mag=5000e-15, eeg=150e-6)
#        self.flat = dict(eog=-1, grad=1e-13, mag=1e-15, eeg=1e-6)




#!/usr/bin/python
# -*- coding: UTF-8 -*-
import unittest
from mock import Mock
from datetime import datetime
import json


class SerializerTests(unittest.TestCase):

    def test_serialize_makes_time_fields_a_string(self):
        from niprov.jsonserializing import JsonSerializer  
        serializer = JsonSerializer()
        record = {}
        record['acquired'] = datetime.now()
        record['created'] = datetime.now()
        out = serializer.serialize(record)
        self.assertEqual(json.loads(out)['acquired'], 
            record['acquired'].isoformat())
        self.assertEqual(json.loads(out)['created'], 
            record['created'].isoformat())

    def test_serialize_makes_args_kwargs_values_strings(self):
        from niprov.jsonserializing import JsonSerializer  
        class CustomType(object):
            pass
        serializer = JsonSerializer()
        record = {}
        ct1 = CustomType()
        ct2 = CustomType()
        record['args'] = [1.23, ct1]
        record['kwargs'] = {'one':ct2, 'two':4.56}
        out = serializer.serialize(record)
        self.assertEqual(json.loads(out)['args'], 
            [1.23, str(ct1)])
        self.assertEqual(json.loads(out)['kwargs'], 
            {'one':str(ct2), 'two':4.56})

    def test_serialize_and_deserialize_dont_balk_if_time_field_absent(self):
        from niprov.jsonserializing import JsonSerializer  
        serializer = JsonSerializer()
        record = {}
        jsonrecord = serializer.serialize(record)
        out = serializer.deserialize(jsonrecord)
        self.assertEqual(record, out)

    def test_deserialize_makes_time_field_a_datetime_object(self):
        from niprov.jsonserializing import JsonSerializer  
        serializer = JsonSerializer()
        record = {}
        acquired = datetime.now()
        record['acquired'] = acquired.isoformat()      
        created = datetime.now()
        record['created'] = created.isoformat()      
        out = serializer.deserialize(json.dumps(record))
        self.assertEqual(out['acquired'], acquired)
        self.assertEqual(out['created'], created)

    def test_serializeList_makes_time_field_a_string(self):
        from niprov.jsonserializing import JsonSerializer  
        serializer = JsonSerializer()
        record = {}
        record['acquired'] = datetime.now()
        record['created'] = datetime.now()
        out = serializer.serializeList([record])
        self.assertEqual(json.loads(out)[0]['acquired'], 
            record['acquired'].isoformat())
        self.assertEqual(json.loads(out)[0]['created'], 
            record['created'].isoformat())





#!/usr/bin/python
# -*- coding: UTF-8 -*-
from mako.lookup import TemplateLookup
import pkg_resources as pkgr
from niprov.exporter import BaseExporter


class HtmlExporter(BaseExporter):

    def __init__(self, form, filesys, listener, externals, **kwargs):
        super(HtmlExporter, self).__init__(form, **kwargs)
        self.filesys = filesys
        self.listener = listener
        self.externals = externals
        templateDir = pkgr.resource_filename('niprov', 'templates')
        self.templates = TemplateLookup([templateDir])

    def exportList(self, provenance):
        """Publish the provenance for several images in an html file and display in Firefox.

        Args:
            provenance (list): List of provenance dictionaries.
        """
        template = self.templates.get_template('list.mako')
        with self.filesys.open('provenance.html','w') as htmlfile:
            htmlfile.write(template.render(provenance=provenance))
        self.externals.run(['firefox', 'provenance.html'])

    def exportSingle(self, img):
        """Publish the provenance for one image in an html file and display in Firefox.

        Args:
            provenance (dict): Provenance for one image file
        """
        template = self.templates.get_template('single.mako')
        with self.filesys.open('provenance.html','w') as htmlfile:
            htmlfile.write(template.render(provenance=img.provenance))
        self.externals.run(['firefox', 'provenance.html'])




import unittest
import mock
from mock import Mock


class addTests(unittest.TestCase):

    def setUp(self):
        self.opts = Mock()
        self.opts.dryrun = False
        self.repo = Mock()
        self.repo.knows.return_value = False
        self.repo.knowsSeries.return_value = False
        self.img = Mock()
        self.fileFactory = Mock()
        self.fileFactory.locatedAt.return_value = self.img
        self.listener = Mock()

    def add(self, path, transient=False):
        from niprov.adding import add
        return add(path, transient=transient, repository=self.repo, 
            opts=self.opts, listener=self.listener, file=self.fileFactory)

    def assertNotCalledWith(self, m, *args, **kwargs):
        c = mock.call(*args, **kwargs)
        assert c not in m.call_args_list, "Unexpectedly found call: "+str(c)

    def test_Returns_provenance_and_status(self):
        new = '/p/f2'
        (provenance, status) = self.add(new)
        self.assertEqual(provenance, self.img.provenance)
        self.assertEqual(status, 'new')

    def test_Sets_transient_flag_if_provided(self):
        (provenance, status) = self.add('/p/f1', transient=True)
        self.fileFactory.locatedAt.assert_called_with('/p/f1', 
            provenance={'transient':True})

    def test_Stores_provenance(self):
        from niprov.adding import add
        new = '/p/f2'
        (provenance, status) = self.add(new)
        self.repo.add.assert_any_call(provenance)

    def test_Creates_ImageFile_object_with_factory(self):
        (provenance, status) = self.add('p/afile.f')
        self.fileFactory.locatedAt.assert_called_with('p/afile.f', 
            provenance={'transient': False})
        self.assertRaises(AssertionError,
            self.fileFactory.locatedAt.assert_any_call,'root/other.file')

    def test_Calls_inspect(self):
        (provenance, status) = self.add('p/afile.f')
        self.img.inspect.assert_called_with()

    def test_Hands_provenance_to_repository(self):
        (provenance, status) = self.add('p/afile.f')
        self.repo.add.assert_any_call(self.img.provenance)

    def test_If_discovers_file_that_is_known_ignore_it(self):
        self.repo.knows.return_value = True
        (provenance, status) = self.add('p/afile.f')
        assert not self.repo.add.called
        self.listener.knownFile.assert_called_with(self.img.path)
        self.assertEqual(status, 'known')

    def test_If_repo_doesnt_know_file_but_knows_series_update_series(self):
        self.repo.knows.return_value = False
        self.repo.knowsSeries.return_value = True
        series = Mock()
        self.repo.getSeries.return_value = series
        (provenance, status) = self.add('p/afile.f')
        series.addFile.assert_called_with(self.img)
        self.repo.update.assert_called_with(series)
        self.listener.fileFoundInSeries.assert_called_with(self.img, series)
        self.assertEqual(status, 'series')

    def test_If_inspect_raises_exceptions_tells_listener_and_doesnt_save(self):
        self.img.inspect.side_effect = IOError
        (provenance, status) = self.add('p/afile.f')
        self.assertNotCalledWith(self.repo.add, self.img.provenance)
        self.listener.fileError.assert_called_with(self.img.path)
        self.assertEqual(status, 'failed')

    def test_If_dryrun_doesnt_talk_to_repo_and_status_is_test(self):
        self.opts.dryrun = True
        (provenance, status) = self.add('p/afile.f')
        assert not self.repo.add.called
        assert not self.repo.update.called
        assert not self.img.inspect.called
        self.assertEqual(status, 'dryrun')



        




from niprov.discovery import discover
from niprov.inspection import inspect
from niprov.reporting import report
from niprov.logging import log
from niprov.recording import record
from niprov.adding import add
from niprov.renaming import renameDicoms
from niprov.approval import (markForApproval, markedForApproval, approve, 
    selectApproved)



#!/usr/bin/python
# -*- coding: UTF-8 -*-
from niprov.commandline import Commandline
from niprov.jsonfile import JsonFile
from niprov.files import FileFactory
from niprov.options import NiprovOptions


def add(filepath, transient=False, opts=NiprovOptions(), listener=Commandline(),
        repository=JsonFile(), file=FileFactory()):
    """
    Simply register the file.

    Inspects the file and makes it known to the provenance data, such that 
    image files can be logged that have been created using this file. 
    Useful also for temporary files.
    
    Example:
        (provenance, status) = niprov.add('/path/to/my.nii')

    Args:
        filepath (str): Path to the newly created file.
        transient (bool, optional): Set this to True to indicate that the file 
            is only temporary and future checks should not expect it to be 
            physically present. Defaults to False, assuming that the file 
            remains.
        opts (NiprovOptions): General settings for niprov. 
            See :py:mod:`niprov.options`

    Returns:
        tuple: Tuple of new provenance and status. Status is a string with one 
            of the following values:
            'new': File was not known yet and has been added.
            'series': The file was deemed part of a series and has been added.
            'failed': There was an error inspecting the file.
            'known': The file was already known to niprov, nothing happened.
            'dryrun': Function called with opts.dryrun, database not touched.
    """
    img = file.locatedAt(filepath, provenance={'transient':transient})
    if opts.dryrun:
        status = 'dryrun'
    elif repository.knows(img):
        listener.knownFile(img.path)
        status = 'known'
    elif repository.knowsSeries(img):
        series = repository.getSeries(img)
        series.addFile(img)
        repository.update(series)
        listener.fileFoundInSeries(img, series)
        status = 'series'
    else:
        try:
            img.inspect()
        except:
            listener.fileError(img.path)
            status = 'failed'
        else:
            repository.add(img.provenance)
            listener.fileFound(img)
            status = 'new'
    return (img.provenance, status)



    



#!/usr/bin/python
# -*- coding: UTF-8 -*-
import hashlib


class Hasher(object):

    blocksize = 512*8*128

    def digest(self, filename):
        """Determine the unique hash digest for a file.

        Note: takes 26s for a 14GB file.

        Args:
            filename (str): Path to the file for which a hash digest should be made.
        """
        hash = hashlib.md5()
        with open(filename, "r+b") as f:
            for block in iter(lambda: f.read(self.blocksize), ""):
                hash.update(block)
        return hash.hexdigest()








from datetime import datetime
from niprov.basefile import BaseFile
from niprov.dependencies import Dependencies


class ParrecFile(BaseFile):

    def __init__(self, fpath, dependencies=Dependencies(), **kwargs):
        super(ParrecFile, self).__init__(fpath, **kwargs)
        self.libs = dependencies

    def inspect(self):
        provenance = super(ParrecFile, self).inspect()
        img = self.libs.nibabel.load(self.path)
        provenance['subject'] = img.header.general_info['patient_name']
        provenance['protocol'] = img.header.general_info['protocol_name']
        acqstring = img.header.general_info['exam_date']
        dateformat = '%Y.%m.%d / %H:%M:%S'
        provenance['acquired'] = datetime.strptime(acqstring, dateformat)
        provenance['dimensions'] = list(img.shape)
        return provenance



#!/usr/bin/python
# -*- coding: UTF-8 -*-
import subprocess

class Externals(object):
    """Utility that wraps the python subprocess module to start other applications
    """

    def run(self, command):
        """Start a subprocess with the command provided.

        Args:
            command (list): A list of command elements.
        """
        output = subprocess.check_output(command)
        return Result(True, output)


class Result(object):

    def __init__(self, succesful, output):
        self.succesful = succesful
        self.output = output



import unittest
from mock import Mock
from datetime import datetime

class InspectionTests(unittest.TestCase):

    def setUp(self):
        self.fileFactory = Mock()
        self.file = Mock()
        self.fileFactory.locatedAt.return_value = self.file

    def test_Opens_file_with_factory_and_inspects_it(self):
        import niprov.inspection
        aPath = 'example.def'
        out = niprov.inspection.inspect(aPath, file=self.fileFactory)
        self.fileFactory.locatedAt.assert_called_with(aPath)
        self.assertEqual(out, self.file.inspect())


       




from niprov.commandline import Commandline
from niprov.dependencies import Dependencies
from niprov.filesystem import Filesystem
from niprov.hashing import Hasher
from niprov.jsonserializing import JsonSerializer


class BaseFile(object):

    def __init__(self, fpath, provenance=None, listener=Commandline(), 
            filesystem=Filesystem(), hasher=Hasher(), 
            serializer=JsonSerializer()):
        self.path = fpath
        if provenance:
            self.provenance = provenance
            if 'path' not in provenance:
                self.provenance['path'] = self.path
        else:
            self.provenance = {'path':self.path}
        self.listener = listener
        self.filesystem = filesystem
        self.hasher = hasher
        self.serializer = serializer

    def inspect(self):
        self.provenance['size'] = self.filesystem.getsize(self.path)
        self.provenance['created'] = self.filesystem.getctime(self.path)
        self.provenance['hash'] = self.hasher.digest(self.path)
        return self.provenance

    def attach(self):
        """
        Attach the current provenance to the file by saving it encoded 
        in a small textfile alongside it.

        The resulting file's name is like the file it describes,
        but with the .provenance extension.
        """
        provstr = self.serializer.serialize(self.provenance)
        self.filesystem.write(self.path+'.provenance', provstr)

    def getSeriesId(self):
        pass



import unittest
from mock import Mock
from datetime import datetime
from tests.test_basefile import BaseFileTests


class CNTTests(BaseFileTests):

    def setUp(self):
        super(CNTTests, self).setUp()
        self.filesys.open.return_value = MockFile()
        from niprov.cnt import NeuroscanFile
        self.constructor = NeuroscanFile
        self.file = NeuroscanFile(self.path, listener=self.log, 
            filesystem=self.filesys, hasher=self.hasher, serializer=self.json)

    def test_Inspect_parses_experimental_basics(self):
        out = self.file.inspect()
        self.assertEqual(out['subject'], 'Jane Doe')
        self.assertEqual(out['dimensions'], [32, 2080])
        self.assertEqual(out['acquired'], datetime(2015,3,9,13,7,3))


class MockFile(object):

    def __init__(self):
        self.cursor = -1

    def __enter__(self):
        return self

    def __exit__(self, *args, **kwargs):
        pass

    def read(self, nbytes):
        self.cursor = self.cursor + 1
        return _data[self.cursor]


_data = [
None,   #h.rev               = self._fread(fid,12,'char')
None,   #h.nextfile          = self._fread(fid,1,'long')
None,   #h.prevfile          = self._fread(fid,1,'ulong')
None,   #h.type              = self._fread(fid,1,'char')
None,   #h.id                = self._fread(fid,20,'char')
None,   #h.oper              = self._fread(fid,20,'char')
None,   #h.doctor            = self._fread(fid,20,'char')
None,   #h.referral          = self._fread(fid,20,'char')
None,   #h.hospital          = self._fread(fid,20,'char')
#h.patient = self._fread(fid,20,'char')
b'Jane Doe\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00',
None,   #h.age               = self._fread(fid,1,'short')
None,   #h.sex               = self._fread(fid,1,'char')
None,   #h.hand              = self._fread(fid,1,'char')
None,   #h.med               = self._fread(fid,20, 'char')
None,   #h.category          = self._fread(fid,20, 'char')
None,   #h.state             = self._fread(fid,20, 'char')
None,   #h.label             = self._fread(fid,20, 'char')
b'09/03/15' ,   #h.date              = self._fread(fid,10, 'char')
b'13:07:03',   #h.time              = self._fread(fid,12, 'char')
None,   #h.mean_age          = self._fread(fid,1,'float')
None,   #h.stdev             = self._fread(fid,1,'float')
None,   #h.n                 = self._fread(fid,1,'short')
None,   #h.compfile          = self._fread(fid,38,'char')
None,   #h.spectwincomp      = self._fread(fid,1,'float')
None,   #h.meanaccuracy      = self._fread(fid,1,'float')
None,   #h.meanlatency       = self._fread(fid,1,'float')
None,   #h.sortfile          = self._fread(fid,46,'char')
None,   #h.numevents         = self._fread(fid,1,'int')
None,   #h.compoper          = self._fread(fid,1,'char')
None,   #h.avgmode           = self._fread(fid,1,'char')
None,   #h.review            = self._fread(fid,1,'char')
None,   #h.nsweeps           = self._fread(fid,1,'ushort')
None,   #h.compsweeps        = self._fread(fid,1,'ushort')
None,   #h.acceptcnt         = self._fread(fid,1,'ushort')
None,   #h.rejectcnt         = self._fread(fid,1,'ushort')
None,   #h.pnts              = self._fread(fid,1,'ushort')
b' \x00',   #h.nchannels         = self._fread(fid,1,'ushort')
None,   #h.avgupdate         = self._fread(fid,1,'ushort')
None,   #h.domain            = self._fread(fid,1,'char')
None,   #h.variance          = self._fread(fid,1,'char')
None,   #h.rate              = self._fread(fid,1,'ushort')
None,   #h.scale             = self._fread(fid,1,'double')
None,   #h.veogcorrect       = self._fread(fid,1,'char')
None,   #h.heogcorrect       = self._fread(fid,1,'char')
None,   #h.aux1correct       = self._fread(fid,1,'char')
None,   #h.aux2correct       = self._fread(fid,1,'char')
None,   #h.veogtrig          = self._fread(fid,1,'float')
None,   #h.heogtrig          = self._fread(fid,1,'float')
None,   #h.aux1trig          = self._fread(fid,1,'float')
None,   #h.aux2trig          = self._fread(fid,1,'float')
None,   #h.heogchnl          = self._fread(fid,1,'short')
None,   #h.veogchnl          = self._fread(fid,1,'short')
None,   #h.aux1chnl          = self._fread(fid,1,'short')
None,   #h.aux2chnl          = self._fread(fid,1,'short')
None,   #h.veogdir           = self._fread(fid,1,'char')
None,   #h.heogdir           = self._fread(fid,1,'char')
None,   #h.aux1dir           = self._fread(fid,1,'char')
None,   #h.aux2dir           = self._fread(fid,1,'char')
None,   #h.veog_n            = self._fread(fid,1,'short')
None,   #h.heog_n            = self._fread(fid,1,'short')
None,   #h.aux1_n            = self._fread(fid,1,'short')
None,   #h.aux2_n            = self._fread(fid,1,'short')
None,   #h.veogmaxcnt        = self._fread(fid,1,'short')
None,   #h.heogmaxcnt        = self._fread(fid,1,'short')
None,   #h.aux1maxcnt        = self._fread(fid,1,'short')
None,   #h.aux2maxcnt        = self._fread(fid,1,'short')
None,   #h.veogmethod        = self._fread(fid,1,'char')
None,   #h.heogmethod        = self._fread(fid,1,'char')
None,   #h.aux1method        = self._fread(fid,1,'char')
None,   #h.aux2method        = self._fread(fid,1,'char')
None,   #h.ampsensitivity    = self._fread(fid,1,'float')
None,   #h.lowpass           = self._fread(fid,1,'char')
None,   #h.highpass          = self._fread(fid,1,'char')
None,   #h.notch             = self._fread(fid,1,'char')
None,   #h.autoclipadd       = self._fread(fid,1,'char')
None,   #h.baseline          = self._fread(fid,1,'char')
None,   #h.offstart          = self._fread(fid,1,'float')
None,   #h.offstop           = self._fread(fid,1,'float')
None,   #h.reject            = self._fread(fid,1,'char')
None,   #h.rejstart          = self._fread(fid,1,'float')
None,   #h.rejstop           = self._fread(fid,1,'float')
None,   #h.rejmin            = self._fread(fid,1,'float')
None,   #h.rejmax            = self._fread(fid,1,'float')
None,   #h.trigtype          = self._fread(fid,1,'char')
None,   #h.trigval           = self._fread(fid,1,'float')
None,   #h.trigchnl          = self._fread(fid,1,'char')
None,   #h.trigmask          = self._fread(fid,1,'short')
None,   #h.trigisi           = self._fread(fid,1,'float')
None,   #h.trigmin           = self._fread(fid,1,'float')
None,   #h.trigmax           = self._fread(fid,1,'float')
None,   #h.trigdir           = self._fread(fid,1,'char')
None,   #h.autoscale         = self._fread(fid,1,'char')
None,   #h.n2                = self._fread(fid,1,'short')
None,   #h.dir               = self._fread(fid,1,'char')
None,   #h.dispmin           = self._fread(fid,1,'float')
None,   #h.dispmax           = self._fread(fid,1,'float')
None,   #h.xmin              = self._fread(fid,1,'float')
None,   #h.xmax              = self._fread(fid,1,'float')
None,   #h.automin           = self._fread(fid,1,'float')
None,   #h.automax           = self._fread(fid,1,'float')
None,   #h.zmin              = self._fread(fid,1,'float')
None,   #h.zmax              = self._fread(fid,1,'float')
None,   #h.lowcut            = self._fread(fid,1,'float')
None,   #h.highcut           = self._fread(fid,1,'float')
None,   #h.common            = self._fread(fid,1,'char')
None,   #h.savemode          = self._fread(fid,1,'char')
None,   #h.manmode           = self._fread(fid,1,'char')
None,   #h.ref               = self._fread(fid,10,'char')
None,   #h.rectify           = self._fread(fid,1,'char')
None,   #h.displayxmin       = self._fread(fid,1,'float')
None,   #h.displayxmax       = self._fread(fid,1,'float')
None,   #h.phase             = self._fread(fid,1,'char')
None,   #h.screen            = self._fread(fid,16,'char')
None,   #h.calmode           = self._fread(fid,1,'short')
None,   #h.calmethod         = self._fread(fid,1,'short')
None,   #h.calupdate         = self._fread(fid,1,'short')
None,   #h.calbaseline       = self._fread(fid,1,'short')
None,   #h.calsweeps         = self._fread(fid,1,'short')
None,   #h.calattenuator     = self._fread(fid,1,'float')
None,   #h.calpulsevolt      = self._fread(fid,1,'float')
None,   #h.calpulsestart     = self._fread(fid,1,'float')
None,   #h.calpulsestop      = self._fread(fid,1,'float')
None,   #h.calfreq           = self._fread(fid,1,'float')
None,   #h.taskfile          = self._fread(fid,34,'char')
None,   #h.seqfile           = self._fread(fid,34,'char')
None,   #h.spectmethod       = self._fread(fid,1,'char')
None,   #h.spectscaling      = self._fread(fid,1,'char')
None,   #h.spectwindow       = self._fread(fid,1,'char')
None,   #h.spectwinlength    = self._fread(fid,1,'float')
None,   #h.spectorder        = self._fread(fid,1,'char')
None,   #h.notchfilter       = self._fread(fid,1,'char')
None,   #h.headgain          = self._fread(fid,1,'short')
None,   #h.additionalfiles   = self._fread(fid,1,'int')
None,   #h.unused            = self._fread(fid,5,'char')
None,   #h.fspstopmethod     = self._fread(fid,1,'short')
None,   #h.fspstopmode       = self._fread(fid,1,'short')
None,   #h.fspfvalue         = self._fread(fid,1,'float')
None,   #h.fsppoint          = self._fread(fid,1,'short')
None,   #h.fspblocksize      = self._fread(fid,1,'short')
None,   #h.fspp1             = self._fread(fid,1,'ushort')
None,   #h.fspp2             = self._fread(fid,1,'ushort')
None,   #h.fspalpha          = self._fread(fid,1,'float')
None,   #h.fspnoise          = self._fread(fid,1,'float')
None,   #h.fspv1             = self._fread(fid,1,'short')
None,   #h.montage           = self._fread(fid,40,'char')
None,   #h.eventfile         = self._fread(fid,40,'char')
None,   #h.fratio            = self._fread(fid,1,'float')
None,   #h.minor_rev         = self._fread(fid,1,'char')
None,   #h.eegupdate         = self._fread(fid,1,'short')
None,   #h.compressed        = self._fread(fid,1,'char')
None,   #h.xscale            = self._fread(fid,1,'float')
None,   #h.yscale            = self._fread(fid,1,'float')
None,   #h.xsize             = self._fread(fid,1,'float')
None,   #h.ysize             = self._fread(fid,1,'float')
None,   #h.acmode            = self._fread(fid,1,'char')
None,   #h.commonchnl        = self._fread(fid,1,'uchar')
None,   #h.xtics             = self._fread(fid,1,'char')
None,   #h.xrange            = self._fread(fid,1,'char')
None,   #h.ytics             = self._fread(fid,1,'char')
None,   #h.yrange            = self._fread(fid,1,'char')
None,   #h.xscalevalue       = self._fread(fid,1,'float')
None,   #h.xscaleinterval    = self._fread(fid,1,'float')
None,   #h.yscalevalue       = self._fread(fid,1,'float')
None,   #h.yscaleinterval    = self._fread(fid,1,'float')
None,   #h.scaletoolx1       = self._fread(fid,1,'float')
None,   #h.scaletooly1       = self._fread(fid,1,'float')
None,   #h.scaletoolx2       = self._fread(fid,1,'float')
None,   #h.scaletooly2       = self._fread(fid,1,'float')
None,   #h.port              = self._fread(fid,1,'short')
b' \x08\x00\x00',   #h.numsamples        = self._fread(fid,1,'ulong')
None,   #h.filterflag        = self._fread(fid,1,'char')
None,   #h.lowcutoff         = self._fread(fid,1,'float')
None,   #h.lowpoles          = self._fread(fid,1,'short')
None,   #h.highcutoff        = self._fread(fid,1,'float')
None,   #h.highpoles         = self._fread(fid,1,'short')
None,   #h.filtertype        = self._fread(fid,1,'char')
None,   #h.filterdomain      = self._fread(fid,1,'char')
None,   #h.snrflag           = self._fread(fid,1,'char')
None,   #h.coherenceflag     = self._fread(fid,1,'char')
None,   #h.continuoustype    = self._fread(fid,1,'char')
None,   #h.eventtablepos     = self._fread(fid,1,'ulong')
None,   #h.continuousseconds = self._fread(fid,1,'float')
None,   #h.channeloffset     = self._fread(fid,1,'long')
None,   #h.autocorrectflag   = self._fread(fid,1,'char')
None,   #h.dcthreshold       = self._fread(fid,1,'uchar')
]





class NiprovOptions(object):
    """General settings for niprov.

    Attributes:
        dryrun (bool): Do not execute commands or make lasting changes to the 
            provenance database. Defaults to False.
        verbose (bool): Output extra information. Defaults to False.
    """
    dryrun = False
    verbose = False



#!/usr/bin/python
# -*- coding: UTF-8 -*-


class Commandline(object):

    def setOptions(self, opts):
        self.opts = opts

    def fileFound(self, image):
        template = '[provenance] {0}'
        print(template.format(image.path))

    def fileFoundInSeries(self, img, series):
        template = '[provenance] Adding {0} file to series: {1}'
        nfiles = len(series.provenance['filesInSeries'])
        print(template.format(ordinal(nfiles), series.getSeriesId()))

    def missingDependencyForImage(self, lib, fpath):
        template = '[provenance] Missing python package "{0}" to read file: {1}'        
        print(template.format(lib, fpath))

    def fileError(self, fpath):
        import traceback
        traceback.print_exc()
        print('[provenance] Error inspecting file: {0}'.format(fpath))

    def interpretedRecording(self, new, transform, parents):
        template = ('[provenance] Recorded the command [{1}] to create [{0}] '+
            'based on [{2}]')
        print(template.format(', '.join(new), transform, ', '.join(parents)))

    def unknownFile(self, fpath):
        print('[provenance] Unknown file: '+fpath)

    def knownFile(self, fpath):
        print('[provenance] File already known: '+fpath)

    def renamedDicom(self, fpath):
        print('[provenance] Renamed dicom file: '+fpath)

    def discoveryFinished(self, nnew, nadded, nfailed, ntotal):
        print('[provenance] Discovered {0} new, added {1} to series, failed to read {2}, '
           'processed {3} total files.'.format(nnew, nadded, nfailed, ntotal))

    def mnefunEventReceived(self, operationName):
        print('[provenance] Mnefun operation: '+operationName)

    def receivedBashCommand(self, command):
        print('[provenance] Recording command: \n'+(' '.join(command)))

    def filesMarkedForApproval(self, images):
        print('[provenance] Files marked for approval:')
        for img in images:
            print(img.path)

SUFFIXES = {1: 'st', 2: 'nd', 3: 'rd'}
def ordinal(num):
    if 10 <= num % 100 <= 20:
        suffix = 'th'
    else:
        # the second parameter is a default.
        suffix = SUFFIXES.get(num % 10, 'th')
    return str(num) + suffix




#!/usr/bin/python
# -*- coding: UTF-8 -*-
from unittest import TestCase
from mock import Mock, sentinel


class DirectExporterTests(TestCase):

    def test_Returns_input(self):
        from niprov.directexporter import DirectExporter
        exporter = DirectExporter()
        self.assertEqual(sentinel.one, exporter.exportSingle(sentinel.one))
        self.assertEqual(sentinel.list, exporter.exportList(sentinel.list))

    def test_For_narrative_form_Returns_narrator_output(self):
        from niprov.directexporter import DirectExporter
        narrator = Mock()
        exporter = DirectExporter(form='narrative',narrator=narrator)
        out = exporter.exportNarrative(sentinel.one)
        narrator.narrate.assert_called_with(sentinel.one)
        self.assertEqual(out, narrator.narrate(sentinel.one))




from struct import unpack
from datetime import datetime, time
from niprov.basefile import BaseFile


class NeuroscanFile(BaseFile):
    """ Support for the Neuroscan .cnt format.
    """

    _datatypes = {'char':1, 'uchar':1, 'long':4,'ulong':4,'short':2,'ushort':2,
        'float':4, 'double':8,'int':4}

    def __init__(self, fpath, **kwargs):
        super(NeuroscanFile, self).__init__(fpath, **kwargs)

    def inspect(self):
        provenance = super(NeuroscanFile, self).inspect()
        header = self._read()
        provenance['subject'] = header.patient.translate(None, '\x00')
        nchannels = unpack('H',header.nchannels)[0]
        numsamples = unpack('I',header.numsamples)[0]
        provenance['dimensions'] = [nchannels, numsamples]
        acqstring = (header.date+' '+header.time).translate(None, '\x00')
        dtformat = '%d/%m/%y %H:%M:%S'
        provenance['acquired'] = datetime.strptime(acqstring, dtformat)
        return provenance

    def _fread(self, fhandle, size, dtype):
        """ Read bytes from file at current cursor position.

        Emulates MATLAB fread().
        """
        precision = self._datatypes[dtype]
        nbytes = size*precision
        return fhandle.read(nbytes)
 
    def _read(self):
        """
        Read the Neuroscan CNT file header.

        This code is loosely based on the MATLAB function loadcnt.m by
        Sean Fitzgibbon and Arnaud Delorme which is available under the GPLv2 
        license. Part of the original copyright statement and license are 
        reproduced below (in the source code).
        """
        # Copyright (C) 2000 Sean Fitzgibbon, <psspf@id.psy.flinders.edu.au>
        # Copyright (C) 2003 Arnaud Delorme, Salk Institute, arno@salk.edu
        #
        # This program is free software; you can redistribute it and/or modify
        # it under the terms of the GNU General Public License as published by
        # the Free Software Foundation; either version 2 of the License, or
        # (at your option) any later version.
        #
        # This program is distributed in the hope that it will be useful,
        # but WITHOUT ANY WARRANTY; without even the implied warranty of
        # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
        # GNU General Public License for more details.
        #
        # You should have received a copy of the GNU General Public License
        # along with this program; if not, write to the Free Software
        # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
        #
        # Revision 1.21  2005/08/16 22:46:55  arno
        # allowing to read event type 3
        class CntHeader(object):
            pass
        h = CntHeader()
        with self.filesystem.open(self.path,'rb') as fid:
            h.rev               = self._fread(fid,12,'char')
            h.nextfile          = self._fread(fid,1,'long')
            h.prevfile          = self._fread(fid,1,'ulong')
            h.type              = self._fread(fid,1,'char')
            h.id                = self._fread(fid,20,'char')
            h.oper              = self._fread(fid,20,'char')
            h.doctor            = self._fread(fid,20,'char')
            h.referral          = self._fread(fid,20,'char')
            h.hospital          = self._fread(fid,20,'char')
            h.patient           = self._fread(fid,20,'char')
            h.age               = self._fread(fid,1,'short')
            h.sex               = self._fread(fid,1,'char')
            h.hand              = self._fread(fid,1,'char')
            h.med               = self._fread(fid,20, 'char')
            h.category          = self._fread(fid,20, 'char')
            h.state             = self._fread(fid,20, 'char')
            h.label             = self._fread(fid,20, 'char')
            h.date              = self._fread(fid,10, 'char')
            h.time              = self._fread(fid,12, 'char')
            h.mean_age          = self._fread(fid,1,'float')
            h.stdev             = self._fread(fid,1,'float')
            h.n                 = self._fread(fid,1,'short')
            h.compfile          = self._fread(fid,38,'char')
            h.spectwincomp      = self._fread(fid,1,'float')
            h.meanaccuracy      = self._fread(fid,1,'float')
            h.meanlatency       = self._fread(fid,1,'float')
            h.sortfile          = self._fread(fid,46,'char')
            h.numevents         = self._fread(fid,1,'int')
            h.compoper          = self._fread(fid,1,'char')
            h.avgmode           = self._fread(fid,1,'char')
            h.review            = self._fread(fid,1,'char')
            h.nsweeps           = self._fread(fid,1,'ushort')
            h.compsweeps        = self._fread(fid,1,'ushort')
            h.acceptcnt         = self._fread(fid,1,'ushort')
            h.rejectcnt         = self._fread(fid,1,'ushort')
            h.pnts              = self._fread(fid,1,'ushort')
            h.nchannels         = self._fread(fid,1,'ushort')
            h.avgupdate         = self._fread(fid,1,'ushort')
            h.domain            = self._fread(fid,1,'char')
            h.variance          = self._fread(fid,1,'char')
            h.rate              = self._fread(fid,1,'ushort')
            h.scale             = self._fread(fid,1,'double')
            h.veogcorrect       = self._fread(fid,1,'char')
            h.heogcorrect       = self._fread(fid,1,'char')
            h.aux1correct       = self._fread(fid,1,'char')
            h.aux2correct       = self._fread(fid,1,'char')
            h.veogtrig          = self._fread(fid,1,'float')
            h.heogtrig          = self._fread(fid,1,'float')
            h.aux1trig          = self._fread(fid,1,'float')
            h.aux2trig          = self._fread(fid,1,'float')
            h.heogchnl          = self._fread(fid,1,'short')
            h.veogchnl          = self._fread(fid,1,'short')
            h.aux1chnl          = self._fread(fid,1,'short')
            h.aux2chnl          = self._fread(fid,1,'short')
            h.veogdir           = self._fread(fid,1,'char')
            h.heogdir           = self._fread(fid,1,'char')
            h.aux1dir           = self._fread(fid,1,'char')
            h.aux2dir           = self._fread(fid,1,'char')
            h.veog_n            = self._fread(fid,1,'short')
            h.heog_n            = self._fread(fid,1,'short')
            h.aux1_n            = self._fread(fid,1,'short')
            h.aux2_n            = self._fread(fid,1,'short')
            h.veogmaxcnt        = self._fread(fid,1,'short')
            h.heogmaxcnt        = self._fread(fid,1,'short')
            h.aux1maxcnt        = self._fread(fid,1,'short')
            h.aux2maxcnt        = self._fread(fid,1,'short')
            h.veogmethod        = self._fread(fid,1,'char')
            h.heogmethod        = self._fread(fid,1,'char')
            h.aux1method        = self._fread(fid,1,'char')
            h.aux2method        = self._fread(fid,1,'char')
            h.ampsensitivity    = self._fread(fid,1,'float')
            h.lowpass           = self._fread(fid,1,'char')
            h.highpass          = self._fread(fid,1,'char')
            h.notch             = self._fread(fid,1,'char')
            h.autoclipadd       = self._fread(fid,1,'char')
            h.baseline          = self._fread(fid,1,'char')
            h.offstart          = self._fread(fid,1,'float')
            h.offstop           = self._fread(fid,1,'float')
            h.reject            = self._fread(fid,1,'char')
            h.rejstart          = self._fread(fid,1,'float')
            h.rejstop           = self._fread(fid,1,'float')
            h.rejmin            = self._fread(fid,1,'float')
            h.rejmax            = self._fread(fid,1,'float')
            h.trigtype          = self._fread(fid,1,'char')
            h.trigval           = self._fread(fid,1,'float')
            h.trigchnl          = self._fread(fid,1,'char')
            h.trigmask          = self._fread(fid,1,'short')
            h.trigisi           = self._fread(fid,1,'float')
            h.trigmin           = self._fread(fid,1,'float')
            h.trigmax           = self._fread(fid,1,'float')
            h.trigdir           = self._fread(fid,1,'char')
            h.autoscale         = self._fread(fid,1,'char')
            h.n2                = self._fread(fid,1,'short')
            h.dir               = self._fread(fid,1,'char')
            h.dispmin           = self._fread(fid,1,'float')
            h.dispmax           = self._fread(fid,1,'float')
            h.xmin              = self._fread(fid,1,'float')
            h.xmax              = self._fread(fid,1,'float')
            h.automin           = self._fread(fid,1,'float')
            h.automax           = self._fread(fid,1,'float')
            h.zmin              = self._fread(fid,1,'float')
            h.zmax              = self._fread(fid,1,'float')
            h.lowcut            = self._fread(fid,1,'float')
            h.highcut           = self._fread(fid,1,'float')
            h.common            = self._fread(fid,1,'char')
            h.savemode          = self._fread(fid,1,'char')
            h.manmode           = self._fread(fid,1,'char')
            h.ref               = self._fread(fid,10,'char')
            h.rectify           = self._fread(fid,1,'char')
            h.displayxmin       = self._fread(fid,1,'float')
            h.displayxmax       = self._fread(fid,1,'float')
            h.phase             = self._fread(fid,1,'char')
            h.screen            = self._fread(fid,16,'char')
            h.calmode           = self._fread(fid,1,'short')
            h.calmethod         = self._fread(fid,1,'short')
            h.calupdate         = self._fread(fid,1,'short')
            h.calbaseline       = self._fread(fid,1,'short')
            h.calsweeps         = self._fread(fid,1,'short')
            h.calattenuator     = self._fread(fid,1,'float')
            h.calpulsevolt      = self._fread(fid,1,'float')
            h.calpulsestart     = self._fread(fid,1,'float')
            h.calpulsestop      = self._fread(fid,1,'float')
            h.calfreq           = self._fread(fid,1,'float')
            h.taskfile          = self._fread(fid,34,'char')
            h.seqfile           = self._fread(fid,34,'char')
            h.spectmethod       = self._fread(fid,1,'char')
            h.spectscaling      = self._fread(fid,1,'char')
            h.spectwindow       = self._fread(fid,1,'char')
            h.spectwinlength    = self._fread(fid,1,'float')
            h.spectorder        = self._fread(fid,1,'char')
            h.notchfilter       = self._fread(fid,1,'char')
            h.headgain          = self._fread(fid,1,'short')
            h.additionalfiles   = self._fread(fid,1,'int')
            h.unused            = self._fread(fid,5,'char')
            h.fspstopmethod     = self._fread(fid,1,'short')
            h.fspstopmode       = self._fread(fid,1,'short')
            h.fspfvalue         = self._fread(fid,1,'float')
            h.fsppoint          = self._fread(fid,1,'short')
            h.fspblocksize      = self._fread(fid,1,'short')
            h.fspp1             = self._fread(fid,1,'ushort')
            h.fspp2             = self._fread(fid,1,'ushort')
            h.fspalpha          = self._fread(fid,1,'float')
            h.fspnoise          = self._fread(fid,1,'float')
            h.fspv1             = self._fread(fid,1,'short')
            h.montage           = self._fread(fid,40,'char')
            h.eventfile         = self._fread(fid,40,'char')
            h.fratio            = self._fread(fid,1,'float')
            h.minor_rev         = self._fread(fid,1,'char')
            h.eegupdate         = self._fread(fid,1,'short')
            h.compressed        = self._fread(fid,1,'char')
            h.xscale            = self._fread(fid,1,'float')
            h.yscale            = self._fread(fid,1,'float')
            h.xsize             = self._fread(fid,1,'float')
            h.ysize             = self._fread(fid,1,'float')
            h.acmode            = self._fread(fid,1,'char')
            h.commonchnl        = self._fread(fid,1,'uchar')
            h.xtics             = self._fread(fid,1,'char')
            h.xrange            = self._fread(fid,1,'char')
            h.ytics             = self._fread(fid,1,'char')
            h.yrange            = self._fread(fid,1,'char')
            h.xscalevalue       = self._fread(fid,1,'float')
            h.xscaleinterval    = self._fread(fid,1,'float')
            h.yscalevalue       = self._fread(fid,1,'float')
            h.yscaleinterval    = self._fread(fid,1,'float')
            h.scaletoolx1       = self._fread(fid,1,'float')
            h.scaletooly1       = self._fread(fid,1,'float')
            h.scaletoolx2       = self._fread(fid,1,'float')
            h.scaletooly2       = self._fread(fid,1,'float')
            h.port              = self._fread(fid,1,'short')
            h.numsamples        = self._fread(fid,1,'ulong')
            h.filterflag        = self._fread(fid,1,'char')
            h.lowcutoff         = self._fread(fid,1,'float')
            h.lowpoles          = self._fread(fid,1,'short')
            h.highcutoff        = self._fread(fid,1,'float')
            h.highpoles         = self._fread(fid,1,'short')
            h.filtertype        = self._fread(fid,1,'char')
            h.filterdomain      = self._fread(fid,1,'char')
            h.snrflag           = self._fread(fid,1,'char')
            h.coherenceflag     = self._fread(fid,1,'char')
            h.continuoustype    = self._fread(fid,1,'char')
            h.eventtablepos     = self._fread(fid,1,'ulong')
            h.continuousseconds = self._fread(fid,1,'float')
            h.channeloffset     = self._fread(fid,1,'long')
            h.autocorrectflag   = self._fread(fid,1,'char')
            h.dcthreshold       = self._fread(fid,1,'uchar')
        return h








class Narrator(object):
    """Generates human-readable text that describes provenance objects.
    """

    def narrate(self, img):
        provenance = img.provenance
        story = ''
        if 'protocol' in provenance:
            story += 'This is a {0} image. '.format(provenance['protocol'])
        if 'acquired' in provenance:
            datestr = provenance['acquired'].strftime('%B %d, %Y').replace(' 0',' ')
            story += 'It was recorded {0}. '.format(datestr)
        if 'subject' in provenance:
            story += "The participant's name is {0}. ".format(
                provenance['subject'])
        if 'size' in provenance:
            sizeInBytes = provenance['size']
            if sizeInBytes > (1000 * 1000):
                unit = 'MB'
                factor = 1000 * 1000
            else:
                unit = 'KB'
                factor = 1000
            size = int(provenance['size']/factor)
            story += "It is {0}{1} in size. ".format(size, unit)
        return story



import os
from niprov.dependencies import Dependencies
from niprov.commandline import Commandline
from basefile import BaseFile
from dcm import DicomFile
from parrec import ParrecFile
from fif import FifFile
from niprov.cnt import NeuroscanFile


class FileFactory(object):
    """Creates customized File objects.

    Based on dependencies (python packages) installed and the filename, will
    return an object that derives from niprov.basefile.BaseFile.
    """

    formats = {'.par':('nibabel', ParrecFile),
               '.dcm':('dicom', DicomFile),
               '.cnt':(None, NeuroscanFile),
               '.fif':('mne',FifFile)}

    def __init__(self, libs=Dependencies(), listener=Commandline()):
        self.libs = libs
        self.listener = listener
    
    def locatedAt(self, path, provenance=None):
        """Return an object to represent the image file at the given path.

        If no specific class is available to handle a file with the filename 
        provided, will default to a BaseFile. Similarly, if missing the 
        dependency to deal with the format, will fall back to a regular 
        BaseFile.

        Args:
            path: Path to the file to represent.

        Returns:
            :class:`.BaseFile`: An object that derives from BaseFile
        """
        extension = os.path.splitext(path)[1].lower()
        if extension not in self.formats:
            return BaseFile(path, provenance=provenance)
        elif not self.libs.hasDependency(self.formats[extension][0]):
            self.listener.missingDependencyForImage(
                self.formats[extension][0], path)
            return BaseFile(path)
        return self.formats[extension][1](path, provenance=provenance)

    def fromProvenance(self, provenance):
        return self.locatedAt(provenance['path'], provenance)







from niprov.filesystem import Filesystem
from jsonserializing import JsonSerializer
from niprov.files import FileFactory
import os


class JsonFile(object):
    """Stores provenance in a local text file encoded as json.
    """

    def __init__(self, filesys=Filesystem(), json=JsonSerializer(), 
            factory=FileFactory()):
        self.filesys = filesys
        self.json = json
        self.factory = factory
        self.datafile = os.path.expanduser(os.path.join('~','provenance.json'))

    def add(self, record):
        """Add the provenance for one file to storage.

        Args:
            record (dict): Provenance for one image file.
        """
        current = self.all()
        current.append(record)
        jsonstr = self.json.serializeList(current)
        with open(self.datafile, 'w') as fp:
            fp.write(jsonstr)

    def update(self, image):
        """Save changed provenance for this file..

        Args:
            image (:class:`.BaseFile`): Image file that has changed.
        """
        current = self.all()
        for r in range(len(current)):
            if current[r]['path'] == image.path:
                current[r] = image.provenance
        jsonstr = self.json.serializeList(current)
        with open(self.datafile, 'w') as fp:
            fp.write(jsonstr)

    def all(self):
        """Retrieve all known provenance from storage.

        Returns:
            list: List of provenance for known files.
        """
        try:
            jsonstr = self.filesys.read(self.datafile)
        except IOError:
            return []
        return self.json.deserializeList(jsonstr)

    def knowsByPath(self, path):
        """Whether the file at this path has provenance associated with it.

        Returns:
            bool: True if provenance is available for that path.
        """
        try:
            self.byPath(path)
        except IndexError:
            return False
        return True

    def knows(self, image):
        """Whether this file has provenance associated with it.

        Returns:
            bool: True if provenance is available for this image.
        """
        try:
            self.byPath(image.path)
        except IndexError:
            return False
        return True

    def knowsSeries(self, image):
        """Whether the series that this file is part of has provenance 
        associated with it.

        Args:
            image (:class:`.BaseFile`): File for which the series is sought.

        Returns:
            bool: True if provenance is available for this series.
        """
        try:
            self.getSeries(image)
        except IndexError:
            return False
        return True

    def byPath(self, path):
        """Get the provenance for a file at the given path. 

        In the case of a dicom series, this returns the provenance for the 
        series.

        Args:
            path (str): File system path to the image file.

        Returns:
            dict: Provenance for one image file.
        """
        for record in self.all():
            if record['path'] == path:
                return self.factory.fromProvenance(record)
            elif 'filesInSeries' in record and path in record['filesInSeries']:
                return self.factory.fromProvenance(record)
        else:
            raise IndexError('No file with that path known.')

    def bySubject(self, subject):
        """Get the provenance for all files of a given participant. 

        Args:
            subject (str): The name or other ID string.

        Returns:
            list: List of provenance for known files imaging this subject.
        """
        all = self.all()
        return [f for f in all if f['subject']==subject]

    def getSeries(self, image):
        """Get the object that carries provenance for the series that the image 
        passed is in. 

        Args:
            image (:class:`.DicomFile`): File that is part of a series.

        Returns:
            :class:`.DicomFile`: Image object that caries provenance for the series.
        """
        if image.getSeriesId() is None:
            raise IndexError('Image has no series id.')
        seriesId = image.getSeriesId()
        for record in self.all():
            if 'seriesuid' in record and record['seriesuid'] == seriesId:
                return self.factory.fromProvenance(record)
        else:
            raise IndexError('No provenance record for that series.')

    def byApproval(self, approvalStatus):
        allRecords = self.all()
        matches = []
        for prov in allRecords:
            if 'approval' in prov:
                if prov['approval'] == approvalStatus:
                    matches.append(prov)
        return [self.factory.fromProvenance(r) for r in matches]

    def updateApproval(self, fpath, approvalStatus):
        img = self.byPath(fpath)
        img.provenance['approval'] = approvalStatus
        self.update(img)
       




import unittest
from mock import Mock
from datetime import datetime
import os
import pkg_resources as pkgr

class HtmlTests(unittest.TestCase):

    def setUp(self):
        import niprov.html
        self.log = Mock()
        self.externals = Mock()
        self.template = Mock()
        self.templateLookup = Mock()
        self.templateLookup.get_template.return_value = self.template
        self.templateLookupConstructor = Mock()
        self.img = Mock()
        self.img.provenance = {}
        niprov.html.TemplateLookup = self.templateLookupConstructor
        niprov.html.TemplateLookup.return_value = self.templateLookup
        (self.filesys, self.filehandle) = self.setupFilesys()
        self.exporter = niprov.html.HtmlExporter(None, self.filesys, self.log, 
            self.externals)

    def test_Looks_for_templates_in_right_place(self):
        self.templateLookupConstructor.assert_called_with(
            [pkgr.resource_filename('niprov', 'templates')])

    def test_ExportList_writes_output_of_renderer_to_file(self):
        self.exporter.exportList([])
        self.filesys.open.assert_called_with('provenance.html','w')
        self.filehandle.write.assert_called_with(self.template.render())

    def test_Export_writes_output_of_renderer_to_file(self):
        self.exporter.exportSingle(self.img)
        self.filesys.open.assert_called_with('provenance.html','w')
        self.filehandle.write.assert_called_with(self.template.render())

    def test_ExportList_uses_list_template(self):
        self.exporter.exportList([])
        self.templateLookup.get_template.assert_called_with('list.mako')

    def test_Export_uses_single_template(self):
        self.exporter.exportSingle(self.img)
        self.templateLookup.get_template.assert_called_with('single.mako')

    def test_Opens_file_created_with_firefox(self):
        self.exporter.exportList([])
        self.externals.run.assert_any_call(['firefox',
            'provenance.html'])

    def setupFilesys(self):
        filesys = Mock()
        filehandle = Mock()
        filehandle.__exit__ = lambda x,y,z,a : x
        filehandle.__enter__ = lambda x : x
        filesys.open.return_value = filehandle
        return (filesys, filehandle)







from datetime import datetime, time
from niprov.basefile import BaseFile
from niprov.dependencies import Dependencies


class DicomFile(BaseFile):

    def __init__(self, fpath, dependencies=Dependencies(), **kwargs):
        super(DicomFile, self).__init__(fpath, **kwargs)
        self.libs = dependencies

    def inspect(self):
        """
        Inspect the DICOM file attributes.

        If a general AcquisitionDateTime attribute is not present, the 
        SeriesDate and SeriesTime will be used to set the :ref:`field-acquired` 
        provenance field.

        Returns:
            dict: Provenance for the inspected file.
        """
        super(DicomFile, self).inspect()
        img = self.libs.dicom.read_file(self.path)
        self.provenance['subject'] = img.PatientID
        self.provenance['protocol'] = img.SeriesDescription
        self.provenance['seriesuid'] = img.SeriesInstanceUID
        self.provenance['filesInSeries'] = [self.path]
        if hasattr(img, 'NumberOfFrames'):
            self.provenance['multiframeDicom'] = True
            nframes = int(img.NumberOfFrames)
        else:
            self.provenance['multiframeDicom'] = False
            nframes = len(self.provenance['filesInSeries'])
        if hasattr(img, 'Rows'):
            self.provenance['dimensions'] = [int(img.Rows), int(img.Columns), 
                nframes]
        if hasattr(img, 'AcquisitionDateTime'):
            acqstring = img.AcquisitionDateTime.split('.')[0]
            dateformat = '%Y%m%d%H%M%S'
            self.provenance['acquired'] = datetime.strptime(acqstring,dateformat)
        else:
            dateformat = '%Y%m%d'
            acqdate = datetime.strptime(img.SeriesDate, dateformat)
            acqtime = datetime.fromtimestamp(float(img.SeriesTime)).time()
            combined = datetime.combine(acqdate, acqtime)
            self.provenance['acquired'] = combined.replace(microsecond=0)
        return self.provenance


    def getSeriesId(self):
        """
        Return the DICOM "SeriesInstanceUID" that all files in this series 
        have in common.

        Returns:
            str: A string uniquely identifying files belonging to this series.
        """
        if not 'seriesuid' in self.provenance:
            self.inspect()
        return self.provenance['seriesuid']

    def addFile(self, img):
        """
        Add a single DICOM file object to this series.

        The file will be stored in provenance in the 'filesInSeries' list.
        """
        self.provenance['filesInSeries'].append(img.path)
        self._updateNfilesDependentFields()

    def _updateNfilesDependentFields(self):
        if (not self.provenance['multiframeDicom']) and 'dimensions' in self.provenance:
            nfiles = len(self.provenance['filesInSeries'])
            self.provenance['dimensions'][2] = nfiles
        
        



import unittest
from mock import Mock

class FileFactoryTests(unittest.TestCase):

    def setUp(self):
        self.log = Mock()
        self.libs = Mock()
        self.libs.hasDependency.return_value = True
        self.log = Mock()

    def test_BaseFile(self):
        from niprov.files import FileFactory
        from niprov.basefile import BaseFile
        factory = FileFactory(libs=self.libs, listener=self.log)
        fileCreated = factory.locatedAt('example.f')
        self.assertIsInstance(fileCreated, BaseFile)

    def test_If_parrec_passed_uses_ParrecFile(self):
        from niprov.files import FileFactory
        from niprov.parrec import ParrecFile
        factory = FileFactory(libs=self.libs, listener=self.log)
        fileCreated = factory.locatedAt('example.PAR')
        self.assertIsInstance(fileCreated, ParrecFile)

    def test_If_dicom_passed_uses_DicomFile(self):
        from niprov.files import FileFactory
        from niprov.dcm import DicomFile
        factory = FileFactory(libs=self.libs, listener=self.log)
        fileCreated = factory.locatedAt('example.dcm')
        self.assertIsInstance(fileCreated, DicomFile)

    def test_If_fif_passed_uses_FifFile(self):
        from niprov.files import FileFactory
        from niprov.fif import FifFile
        factory = FileFactory(libs=self.libs, listener=self.log)
        fileCreated = factory.locatedAt('example.fif')
        self.assertIsInstance(fileCreated, FifFile)

    def test_If_dcm_passed_but_pydicom_not_installed_tells_listener(self):
        from niprov.files import FileFactory
        from niprov.basefile import BaseFile
        self.libs.hasDependency.return_value = False
        factory = FileFactory(libs=self.libs, listener=self.log)
        fileCreated = factory.locatedAt('example.dcm')
        self.assertIsInstance(fileCreated, BaseFile)
        self.log.missingDependencyForImage.assert_called_with('dicom','example.dcm')

    def test_If_parrec_passed_but_nibabel_not_installed_tells_listener(self):
        from niprov.files import FileFactory
        from niprov.basefile import BaseFile
        self.libs.hasDependency.return_value = False
        factory = FileFactory(libs=self.libs, listener=self.log)
        fileCreated = factory.locatedAt('example.PAR')
        self.assertIsInstance(fileCreated, BaseFile)
        self.log.missingDependencyForImage.assert_called_with('nibabel','example.PAR')

    def test_If_fif_passed_but_mne_not_installed_tells_listener(self):
        from niprov.files import FileFactory
        from niprov.basefile import BaseFile
        self.libs.hasDependency.return_value = False
        factory = FileFactory(libs=self.libs, listener=self.log)
        fileCreated = factory.locatedAt('example.fif')
        self.assertIsInstance(fileCreated, BaseFile)
        self.log.missingDependencyForImage.assert_called_with('mne','example.fif')

    def test_If_extensions_are_not_case_sensitive(self):
        from niprov.files import FileFactory
        from niprov.dcm import DicomFile
        factory = FileFactory(libs=self.libs, listener=self.log)
        fileCreated = factory.locatedAt('example.dCm')
        self.assertIsInstance(fileCreated, DicomFile)

    def test_fromProvenance_inserts_existing_provenance(self):
        import niprov.files
        niprov.files.BaseFile = Mock()
        niprov.files.DicomFile = Mock()
        niprov.files.FileFactory.formats['.dcm'] = ('dicom', niprov.files.DicomFile)
        factory = niprov.files.FileFactory(libs=self.libs, listener=self.log)
        inProvenance = {'path':'some.file','aproperty':'avalue'}
        inProvenanceDcm = {'path':'some.dcm','aproperty':'avalue'}
        fileCreated = factory.fromProvenance(inProvenance)
        niprov.files.BaseFile.assert_called_with('some.file', provenance=inProvenance)
        fileCreated = factory.fromProvenance(inProvenanceDcm)
        niprov.files.DicomFile.assert_called_with('some.dcm', provenance=inProvenanceDcm)

    def test_If_cnt_passed_uses_NeuroscanFile(self):
        from niprov.files import FileFactory
        from niprov.cnt import NeuroscanFile
        factory = FileFactory(libs=self.libs, listener=self.log)
        fileCreated = factory.locatedAt('example.cnt')
        self.assertIsInstance(fileCreated, NeuroscanFile)







import unittest
from mock import Mock
from datetime import datetime


class NarratorTests(unittest.TestCase):


    def test_File(self):
        from niprov.narrator import Narrator
        img = Mock()
        p = {}
        p['protocol'] = 'T1'
        p['acquired'] = datetime(2013, 9, 7, 11, 27, 34)
        p['subject'] = 'John Doe'
        p['size'] = 345123
        img.provenance = p
        storyteller = Narrator()
        self.assertEqual(storyteller.narrate(img), ("This is a T1 image. "
            "It was recorded September 7, 2013. "
            "The participant's name is John Doe. "
            "It is 345KB in size. "))
        



#!/usr/bin/python
# -*- coding: UTF-8 -*-
from niprov.exporter import BaseExporter


class DirectExporter(BaseExporter):
    """Dummy  Exporter class which simply returns the provenance passed.
    """

    def exportList(self, provenance):
        return provenance

    def exportSingle(self, provenance):
        return provenance

    def exportNarrative(self, provenance):
        return self.narrator.narrate(provenance)



import unittest
from mock import Mock


class ApprovalTests(unittest.TestCase):

    def setUp(self):
        self.repo = Mock()
        self.listener = Mock()

    def test_markForApproval_tells_repo_to_set_approval_to_pending(self):
        import niprov
        niprov.markForApproval(['f1','f2'], repository=self.repo)
        self.repo.updateApproval.assert_any_call('f1','pending')
        self.repo.updateApproval.assert_any_call('f2','pending')

    def test_Approve_tells_repo_to_set_approval_to_granted(self):
        import niprov
        niprov.approve('fx12', repository=self.repo)
        self.repo.updateApproval.assert_any_call('fx12','granted')

    def test_markedForApproval_lists_files_marked(self):
        import niprov
        markedFiles = niprov.markedForApproval(repository=self.repo,
            listener=self.listener)
        self.repo.byApproval.assert_called_with('pending')
        self.assertEqual(self.repo.byApproval(), markedFiles)

    def test_markedForApproval_tells_listener_about_files(self):
        import niprov
        markedFiles = niprov.markedForApproval(repository=self.repo,
            listener=self.listener)
        self.listener.filesMarkedForApproval.assert_called_with(
            markedFiles)

    def test_selectApproved(self):
        import niprov
        a1 = self.mockImg()
        a1.provenance['approval'] = 'granted'
        a2 = self.mockImg()
        a2.provenance['approval'] = 'granted'
        b1 = self.mockImg()
        c1 = self.mockImg()
        c1.provenance['approval'] = 'pending'
        self.repo.byPath.side_effect = lambda p: {'a1':a1,'b1':b1,'a2':a2,'c1':c1}[p]
        selected = niprov.selectApproved(['a1','b1','a2','c1'], 
            repository=self.repo)
        self.assertEqual(selected, [a1.path, a2.path])

    def mockImg(self):
        img = Mock()
        img.provenance = {}
        return img




import unittest
import os, shutil


class ApiTests(unittest.TestCase):

    def setUp(self):
        self.dbpath = os.path.expanduser(os.path.join('~','provenance.json'))
        if os.path.exists(self.dbpath):
            shutil.move(self.dbpath, self.dbpath.replace('.json','.backup.json'))
        os.mkdir('temp')

    def tearDown(self):
        if os.path.exists(self.dbpath):
            shutil.move(self.dbpath, self.dbpath.replace('.json','.test.json'))
        shutil.rmtree('temp')

    def test_Discover(self):
        import niprov
        niprov.discover('testdata')
        img = niprov.report(forFile='testdata/dicom/T1.dcm')
        self.assertEqual(img.provenance['dimensions'], [80, 80, 10])
        img = niprov.report(forFile='testdata/eeg/stub.cnt')
        self.assertEqual(img.provenance['subject'], 'Jane Doe')

    def test_Export_terminal(self):
        import niprov
        niprov.discover('testdata')
        niprov.report(medium='stdout')
        niprov.report(medium='stdout', forFile='testdata/dicom/T1.dcm')

    def test_Log(self):
        import niprov
        niprov.discover('testdata')
        newfile = 'temp/smoothed.test'
        self.touch(newfile)
        niprov.log(newfile, 'test', 'testdata/eeg/stub.cnt')
        img = niprov.report(forFile=newfile)
        self.assertEqual(img.provenance['subject'], 'Jane Doe')
        self.assertEqual(img.provenance['size'], os.path.getsize(newfile))

    def test_Narrative_file(self):
        import niprov
        niprov.discover('testdata')
        text = niprov.report(form='narrative', forFile='testdata/dicom/T1.dcm')
        self.assertEqual(text, ("This is a T1 image. It was recorded August 5, " 
            "2014. The participant's name is 05aug14test. It is 158KB in size. "))

    def test_Approval(self):
        import niprov
        niprov.discover('testdata')
        niprov.markForApproval(['testdata/dicom/T1.dcm',
                                'testdata/dicom/T2.dcm'])
        imgs = niprov.markedForApproval()
        niprov.approve('testdata/dicom/T1.dcm')
        imgs = niprov.markedForApproval()

#    def test_Narrative_pipeline(self):
#        import niprov
#        niprov.discover('testdata')
#        self.touch('temp/smoothed.test')
#        niprov.log('temp/smoothed.test','spatial smoothing',
#            'testdata/parrec/T2.PAR')
#        self.touch('temp/stats.test')
#        niprov.log('temp/stats.test','t-test','temp/smoothed.test')
#        text = niprov.report(format='narrative', forPipeline='temp/stats.test')
#        self.assertEqual(text, ("A T2 image was recorded. A spatial smoothing "
#            "was then applied. A t-test was then applied."))

    def test_Rename(self):
        try:
            import niprov
            files, directory = self.createExtensionlessFiles()
            niprov.renameDicoms(directory)
            assert not os.path.isfile(files[0])
            assert os.path.isfile(files[0]+'.dcm')
        finally:
            self.clearDicomfiles()

    def createExtensionlessFiles(self):
        if not os.path.exists('dicomdir'):
            os.makedirs('dicomdir')
        files = []
        for n in range(5):
            f = os.path.join('dicomdir',str(n))
            files.append(f)
            with open(f,'w') as fhandle:
                fhandle.write('x')
        return files, 'dicomdir'

    def clearDicomfiles(self):
        shutil.rmtree('dicomdir')

    def touch(self, path):
        with open(path,'w') as tempfile:
            tempfile.write('0')





from niprov.narrator import Narrator


class BaseExporter(object):
    """Parent Exporter class from which exporters for specific mediums are 
    derived.
    """

    def __init__(self, form=None, narrator=Narrator()):
        self.form = form
        self.narrator = narrator

    def export(self, provenance):
        """Publish provenance.

        This determines if the provenance is for a single file or multiple,
        and then calls the appropriate more specific export method.
        """
        if self.form == 'narrative':
            return self.exportNarrative(provenance)
        elif isinstance(provenance, list):
            return self.exportList(provenance)
        else:
            return self.exportSingle(provenance)

    def exportList(self, provenance):
        raise NotImplementedError('exportList')

    def exportSingle(self, provenance):
        raise NotImplementedError('exportSingle')

    def exportNarrative(self, provenance):
        raise NotImplementedError('exportNarrative')



import unittest
import mock
from mock import Mock
import os.path as ospath

class DiscoveryTests(unittest.TestCase):

    def setUp(self):
        self.filesys = Mock()
        self.filesys.walk.return_value = [('root',[],['known','new','unknown'])]
        self.listener = Mock()
        self.filt = Mock()
        self.add = Mock()
        self.add.return_value = (Mock(), 'new')

    def discover(self, path):
        import niprov.discovery
        niprov.discovery.add = self.add
        niprov.discovery.discover(path, filesys=self.filesys, 
            listener=self.listener, filefilter=self.filt)

    def test_Calls_add_on_files_encountered(self):
        self.setupFilter('.x')
        self.filesys.walk.return_value = [('root',[],['p/f1.x','p/f2.x']),
            ('root',[],['p/p2/f3.x'])] #(dirpath, dirnames, filenames)
        self.discover('root')
        self.filesys.walk.assert_called_with('root')
        self.add.assert_any_call('root/p/f1.x', transient=False)
        self.add.assert_any_call('root/p/f2.x', transient=False)
        self.add.assert_any_call('root/p/p2/f3.x', transient=False)

    def test_file_filters(self):
        self.setupFilter('valid.file')
        self.filesys.walk.return_value = [('root',[],['valid.file','other.file'])]
        self.discover('root')
        self.add.assert_any_call('root/valid.file', transient=False)
        self.assertNotCalledWith(self.add, 'root/other.file')

    def test_Gives_listener_summary(self):
        self.filesys.walk.return_value = [('root',[],['a','b','c','d','e','f','g','h'])]
        a,b,c,d,e,f,g,h = [(Mock(), s) for s in ['new','new','series','series',
            'series','failed','known','known']]
        self.add.side_effect = [a,b,c,d,e,f,g,h]
        self.discover('root')
        self.listener.discoveryFinished.assert_called_with(nnew=2, nadded=3, nfailed=1, ntotal=8)

    def assertNotCalledWith(self, m, *args, **kwargs):
        c = mock.call(*args, **kwargs)
        assert c not in m.call_args_list, "Unexpectedly found call: "+str(c)

    def setupFilter(self, valid):
        def filter_side_effect(*args):
            if valid in args[0]:
                return True
            return False
        self.filt.include = Mock(side_effect=filter_side_effect)
        




import unittest
from mock import Mock
from datetime import datetime as dt


class loggingTests(unittest.TestCase):

    def setUp(self):
        self.repo = Mock()
        self.repo.knowsByPath.return_value = True
        self.opts = Mock()
        self.opts.dryrun = False
        img = Mock()
        img.provenance = {'acquired':dt.now(),'subject':'JD', 'protocol':'X'}
        self.repo.byPath.return_value = img
        self.filesys = Mock()
        self.listener = Mock()
        self.factory = Mock()
        self.newimg = Mock()
        self.provenancesCreated = []
        def wrapProv(p):
            self.provenancesCreated.append(p)
            return self.newimg
        self.factory.fromProvenance.side_effect = lambda p : wrapProv(p)

    def log(self, *args, **kwargs):
        from niprov.logging import log
        return log(*args, repository=self.repo, filesys=self.filesys, 
            listener=self.listener, factory=self.factory, opts=self.opts, 
            **kwargs)

    def test_Returns_img(self):
        parents = ['/p/f1']
        new = '/p/f2'
        trans = 'Something cool'
        out = self.log(new, trans, parents)
        self.assertEqual(out, self.newimg)

    def test_Stores_provenance(self):
        provenance = self.log('new', 'trans', 'old')
        self.repo.add.assert_any_call(self.newimg.provenance)

    def test_Copies_fields_from_known_parent(self):
        parent = '/p/f1'
        parents = [parent]
        parentProv = {'acquired':dt.now(),'subject':'JB','protocol':'T3'}
        parentImg = Mock()
        parentImg.provenance = parentProv
        self.repo.byPath.side_effect = lambda x: {parent:parentImg}[x]
        self.log('new', 'trans', parents)
        self.assertEqual(self.provenancesCreated[0]['acquired'], parentProv['acquired'])
        self.assertEqual(self.provenancesCreated[0]['subject'], parentProv['subject'])
        self.assertEqual(self.provenancesCreated[0]['protocol'], parentProv['protocol'])

    def test_Adds_code_or_logtext(self):
        self.log('new', 'trans', 'old', code='abc', logtext='def')
        self.assertEqual(self.provenancesCreated[0]['code'],'abc')
        self.assertEqual(self.provenancesCreated[0]['logtext'],'def')

    def test_Accepts_temp_flag(self):
        parents = ['/p/f1']
        new = '/p/f2'
        trans = 'Something cool'
        self.log(new, trans, parents, transient=True)
        self.assertEqual(self.provenancesCreated[0]['transient'], True)

    def test_If_file_doesnt_exists_tells_listener_and_doesnt_save_prov(self):
        self.filesys.fileExists.return_value = False
        parents = ['/p/f1']
        new = '/p/f2'
        trans = 'Something cool'
        self.assertRaises(IOError, self.log, new, trans, parents)
        assert not self.repo.add.called

    def test_For_nonexisting_transient_file_behaves_normal(self):
        self.filesys.fileExists.return_value = False
        parents = ['/p/f1']
        new = '/p/f2'
        trans = 'Something cool'
        self.log(new, trans, parents, transient=True)
        self.assertEqual(self.provenancesCreated[0]['transformation'], trans)

    def test_Can_pass_multiple_new_files(self):
        parents = ['/p/f1']
        new = ['/p/f2','/p/f3']
        trans = 'Something cool'
        self.log(new, trans, parents)
        self.assertEqual(self.provenancesCreated[0]['parents'], parents)
        self.assertEqual(self.provenancesCreated[1]['parents'], parents)
        self.assertEqual(self.provenancesCreated[0]['path'], new[0])
        self.assertEqual(self.provenancesCreated[1]['path'], new[1])

    def test_Script_added_to_provenance(self):
        parents = ['/p/f1']
        new = '/p/f2'
        trans = 'Something cool'
        script = '/p/test.py'
        self.log(new, trans, parents, script=script)
        self.assertEqual(self.provenancesCreated[0]['script'], script)

    def test_Accepts_and_processes_custom_provenance(self):
        parents = ['/p/f1']
        new = '/p/f2'
        trans = 'Something cool'
        p = {'akey':'avalue'}
        self.log(new, trans, parents, provenance=p)
        self.assertEqual(self.provenancesCreated[0]['akey'], 'avalue')

    def test_Notifies_listener_and_exits_if_parent_unknown(self):
        self.repo.knowsByPath.return_value = False
        parent = '/p/f1'
        parents = [parent]
        provenance = self.log('new', 'trans', parents)
        self.listener.unknownFile.assert_called_with(parent)
        assert not self.repo.add.called

    def test_Doesnt_complain_if_parent_is_missing_basic_fields(self):
        img = Mock()
        img.provenance = {'acquired':dt.now()} #missing subject
        self.repo.byPath.return_value = img
        provenance = self.log('new', 'trans', ['/p/f1parent'])
        self.assertNotIn('subject', self.provenancesCreated[0])

    def test_Inspects_nontransient_new_files(self):
        parents = ['/p/f1']
        new = ['/p/f2','/p/f3']
        trans = 'Something cool'
        provenance = self.log(new, trans, parents, transient=True)
        assert not self.newimg.inspect.called
        provenance = self.log(new, trans, parents)
        self.newimg.inspect.assert_called_with()

    def test_On_dryrun_all_new_files_are_transient(self):
        parents = ['/p/f1']
        new = ['/p/f2','/p/f3']
        trans = 'Something cool'
        self.opts.dryrun = True
        provenance = self.log(new, trans, parents)
        assert not self.newimg.inspect.called
        self.assertEqual(self.provenancesCreated[0]['transient'], True)

    def test_On_dryrun_provenance_not_saved_to_repo(self):
        self.opts.dryrun = True
        provenance = self.log('new', 'trans', 'old')
        assert not self.repo.add.called




import unittest
from mock import Mock
from datetime import datetime
from tests.test_basefile import BaseFileTests


class FifTests(BaseFileTests):

    def setUp(self):
        super(FifTests, self).setUp()
        self.libs = Mock()
        self.setupMne()
        from niprov.fif import FifFile
        self.constructor = FifFile
        self.ckwargs = {'dependencies':self.libs}
        self.file = FifFile(self.path, listener=self.log, 
            filesystem=self.filesys, hasher=self.hasher, dependencies=self.libs,
            serializer=self.json)

    def test_Gets_basic_info_from_mne_and_returns_it(self):
        out = self.file.inspect()
        self.assertEqual(out['subject'], 'John Doeish')
        self.assertEqual(out['project'], 'worlddom')
        self.assertEqual(out['acquired'], self.acquired)

    def test_Gets_dimensions(self):
        out = self.file.inspect()
        self.assertEqual(out['dimensions'], [123, 91])

    def test_Attach_method(self):
        self.file.provenance = {'foo':'bar'}
        self.json.serialize.side_effect = lambda p: str(p)
        self.file.attach()
        self.assertIn('existing bla bla ', #original string+added space 
            self.img.info['description'])
        self.assertIn('NIPROV:'+str(self.file.provenance), 
            self.img.info['description'])
        self.libs.mne.io.write_info.assert_called_with(self.file.path, 
            self.img.info)

    def setupMne(self):
        TS = 1422522595.76096
        self.acquired = datetime.fromtimestamp(TS)
        self.img = Mock()
        self.img.info = {
            'meas_date':(TS,),
            'proj_name':'worlddom',
            'subject_info':{'first_name':'John','last_name':'Doeish'},
            'nchan':123,
            'sfreq':200,
            'description':'existing bla bla'}
        self.img.first_samp = 10
        self.img.last_samp = 100
        self.libs.mne.io.Raw.return_value = self.img
        self.libs.mne.io.read_info.return_value = self.img.info
        self.libs.hasDependency.return_value = True






#!/usr/bin/python
# -*- coding: UTF-8 -*-
from niprov.externals import Externals
from niprov.logging import log
from niprov.commandline import Commandline
from niprov.options import NiprovOptions
from capturing import OutputCapture


def record(command, new=None, parents=None, transient=False, args=None, 
    kwargs=None, externals=Externals(), listener=Commandline(), 
    opts=NiprovOptions()):
    """Execute a command and log it as provenance for the newly created file.

    Args:
        command (list or str or callable): Command to be executed. Either a 
            string of executable system code, a list of components thereof, or 
            a python function object. 
        new (list or str, optional): Override path to the new file(s), i.e. if 
            they cannot be parsed from the command.
        parents (list or str, optional): Override paths to parent file(s), i.e. 
            if they cannot be parsed from the command.
        transient (bool, optional): Set this to True to indicate that the file 
            is only temporary and future checks should not expect it to be 
            physically present. Defaults to False, assuming that the file 
            remains.
        args (list, optional): Positional arguments to be passed to command.
        kwargs (dict, optional): Keyword arguments to be passed to command.
        opts (NiprovOptions): General settings for niprov. 
            See :py:mod:`niprov.options`

    Returns:
        dict: New provenance
    """
    listener.setOptions(opts)

    # initialize mutable defaults:
    if new is None:
        new = []
    if parents is None:
        parents = []
    if args is None:
        args = []
    if kwargs is None:
        kwargs = {}

    # initialize singular/plural ducktyping
    if isinstance(new, basestring):
        new = [new]
    if isinstance(parents, basestring):
        parents = [parents]
    if isinstance(command, basestring):
        command = command.split()

    # gather tranformation provenance
    provenance = {}
    if isinstance(command, (list, tuple)): #command bash command list
        listener.receivedBashCommand(command)
        transformation = command[0]
        code = ' '.join(command)
        script = None
        parsingNew = len(new) == 0
        parsingParents = len(parents) == 0
        for c in range(len(command)):
            if command[c] in ['-out','-o'] and parsingNew:
                new.append(command[c+1])
            if command[c] in ['-in','-i'] and parsingParents:
                parents.append(command[c+1])
    else:                                   #command is python code
        transformation = command.func_name
        script = command.func_code.co_filename
        code = None
        provenance['args'] = args
        provenance['kwargs'] = kwargs
    listener.interpretedRecording(new, transformation, parents)

    # run transformation
    if opts.dryrun:
        output = None
    else:
        if isinstance(command, (list, tuple)):
            result = externals.run(command)
            output = result.output
        else:
            with OutputCapture() as captured:
                command(*args, **kwargs)
            output = captured.output

    # defer the rest to log()
    return log(new, transformation, parents, code=code, transient=transient,
        logtext=output, script=script, provenance=provenance, opts=opts)





#!/usr/bin/python
# -*- coding: UTF-8 -*-
from __future__ import print_function
import copy
from niprov.exporter import BaseExporter


class StandardOutputExporter(BaseExporter):

    _expectedFields = ['acquired','subject','protocol','dimensions','path']

    def exportList(self, provenance):
        """Publish the provenance for several images on the commandline.

        Args:
            provenance (list): List of provenance dictionaries.
        """
        print('\n')
        print('{0:20} {1:12} {2:24} {3:20} {4:24}'.format(*self._expectedFields))
        for record in provenance:
            self.exportSummary(record)
        print('\n')

    def exportSingle(self, img):
        """Publish the provenance for one image on the commandline.

        Args:
            provenance (dict): Provenance for one image file
        """
        print('\n')
        for field, value in img.provenance.items():
            print('{0:24} {1}'.format(field+':', str(value)))
        print('\n')

    def exportSummary(self, provenance):
        """Publish a summary of the provenance for one image as one line in 
        the terminal.

        Args:
            provenance (dict): Provenance for one image file
        """
        provcopy = copy.deepcopy(provenance)
        for field in self._expectedFields:
            if field not in provcopy:
                provcopy[field] = None
        tmp = ('{0[acquired]!s}  {0[subject]!s:12} {0[protocol]!s:24} '
            '{0[dimensions]!s:20} {0[path]!s:24}')
        print(tmp.format(provcopy))

    def exportNarrative(self, provenance):
        """Publish provenance in a 'story' format in the terminal.

        Args:
            provenance (dict or list): Provenance either for one file or several.
        """
        print('\n')
        print(self.narrator.narrate(provenance))
        print('\n')



#!/usr/bin/python
# -*- coding: UTF-8 -*-
from niprov.filesystem import Filesystem
from niprov.commandline import Commandline
from niprov.html import HtmlExporter
from niprov.stdout import StandardOutputExporter
from niprov.directexporter import DirectExporter
from niprov.externals import Externals


class ExportFactory(object):

    def __init__(self):
        self.listener = Commandline()
        self.filesys = Filesystem()

    def createExporter(self, medium, form):
        """
        Return an object that can publish provenance in a specific medium.

        Args:
            medium (str): One of 'stdout' or 'html' or None.
            form (str): One of 'narrative' or None
        """
        if medium is None:
            return DirectExporter(form)
        elif medium == 'html':
            return HtmlExporter(form, self.filesys, self.listener, Externals())
        elif medium == 'stdout':
            return StandardOutputExporter(form)
        else:
            raise ValueError('Unknown medium: '+str(medium))




import unittest
from mock import Mock
from datetime import datetime
from tests.test_basefile import BaseFileTests


class NiftiTests(BaseFileTests):

    def setUp(self):
        super(NiftiTests, self).setUp()
        self.libs = Mock()
        self.img = Mock()
        self.hdr = Mock()
        self.setupNibabel()
        from niprov.nifti import NiftiFile
        self.constructor = NiftiFile
        self.file = NiftiFile(self.path, listener=self.log, 
            filesystem=self.filesys, hasher=self.hasher, dependencies=self.libs,
            serializer=self.json)

    def setupNibabel(self):
        # Extension constructor simply creates a tuple of arguments
        self.img.get_header.return_value = self.hdr
        self.libs.nibabel.nifti1.Nifti1Extension = lambda x,y: ('extension',x,y)
        self.libs.nibabel.load.return_value = self.img
        self.libs.hasDependency.return_value = True

    def test_Attach_method(self):
        self.file.provenance = {'foo':'bar'}
        self.file.attach()
        self.hdr.extensions.append.assert_called_with(('extension','comment', 
            self.json.serialize(self.file.provenance)))
        self.img.to_filename.assert_called_with(self.file.path)




#!/usr/bin/python
# -*- coding: UTF-8 -*-
from setuptools import setup, find_packages
import os

try:
   import pypandoc
   README = pypandoc.convert('README.md', 'rst')
except (IOError, ImportError):
   README = ''

setup(name='niprov',
      version='0.1.post13',
      author='Jasper J.F. van den Bosch',
      author_email='japsai@gmail.com',
      description='provenance for neuroimaging data',
      packages=find_packages(),
      url = 'https://github.com/ilogue/niprov',
      test_suite="tests",
      scripts=['executables/provenance'],
      zip_safe=False,
      license='BSD',
      long_description=README,
      classifiers=[
            'License :: OSI Approved :: BSD License',
            'Intended Audience :: Science/Research',
            'Topic :: Scientific/Engineering'],
      package_data={'niprov': ['discovery-filter.txt']},
      include_package_data=True,
      install_requires=['mako']
      )



#!/usr/bin/python
# -*- coding: UTF-8 -*-
import json
import copy
from datetime import datetime


class JsonSerializer(object):
    """Helper to convert provenance data to and from json encoded strings.
    """

    datetimeFields = ['acquired','created']

    def serialize(self, record):
        """
        Convert one provenance item from its native python dict type to
        a json string.

        Args:
            record (dict): The provenance item to convert.

        Returns:
            str: Json version of the provenance.
        """
        return json.dumps(self._deflate(record))

    def deserialize(self, jsonRecord):
        """
        Convert one provenance item from its json string version to the 
        native python dictionary format.

        Args:
            jsonRecord (str): The provenance item to convert as json-encoded 
                string.

        Returns:
            dict: Python dictionary of the provenance.
        """
        return self._inflate(json.loads(jsonRecord))

    def serializeList(self, listOfRecords):
        """
        Convert a list of provenance items from its native list of python dict 
        type to a json string.

        Args:
            listOfRecords (list): The provenance items to convert.

        Returns:
            str: Json version of the provenance items.
        """
        flatRecords = [self._deflate(r) for r in listOfRecords]
        return json.dumps(flatRecords)

    def deserializeList(self, jsonListOfRecords):
        """
        Convert a list of provenance items from its json string version to the 
        a list of the native python dictionary format.

        Args:
            jsonListOfRecords (str): The provenance items to convert as 
                json-encoded string.

        Returns:
            list: Python list of dictionaries of the provenance.
        """
        flatRecords = json.loads(jsonListOfRecords)
        return [self._inflate(r) for r in flatRecords]

    def _deflate(self, record):
        isoformat = "%Y-%m-%dT%H:%M:%S.%f"
        flatRecord = copy.deepcopy(record)
        for field in self.datetimeFields:
            if field in record:
                flatRecord[field] = record[field].strftime(isoformat)
        if 'args' in record:
            flatRecord['args'] = [self._strcust(a) for a in record['args']]
        if 'kwargs' in record:
            kwargs = record['kwargs']
            flatRecord['kwargs'] = {k: self._strcust(kwargs[k]) 
                for k in kwargs.keys()}
        return flatRecord

    def _inflate(self, flatRecord):
        isoformat = "%Y-%m-%dT%H:%M:%S.%f"
        record = flatRecord
        for field in self.datetimeFields:
            if field in record:
                record[field] = datetime.strptime(record[field], isoformat)
        return record

    def _strcust(self, val):
        """Stringify an object that is not of a simple type."""
        if not isinstance(val, (str, unicode, int, float, bool, type(None))):
            return str(val)
        return val



#!/usr/bin/python
# -*- coding: UTF-8 -*-
from unittest import TestCase
from mock import Mock, sentinel, patch


class StdoutExporterTests(TestCase):

    def test_For_narrative_form_Returns_narrator_output(self):
        mprint = Mock()
        from niprov.stdout import StandardOutputExporter
        narrator = Mock()
        with patch('__builtin__.print') as mprint:
            exporter = StandardOutputExporter(form='narrative', 
                narrator=narrator)
            out = exporter.exportNarrative(sentinel.one)
            narrator.narrate.assert_called_with(sentinel.one)
            mprint.assert_any_call(narrator.narrate())




import unittest
from mock import Mock
from datetime import datetime
from tests.test_basefile import BaseFileTests


class ParrecTests(BaseFileTests):

    def setUp(self):
        super(ParrecTests, self).setUp()
        self.libs = Mock()
        self.setupNibabel()
        from niprov.parrec import ParrecFile
        self.constructor = ParrecFile
        self.ckwargs = {'dependencies':self.libs}
        self.file = ParrecFile(self.path, listener=self.log, 
            filesystem=self.filesys, hasher=self.hasher, dependencies=self.libs,
            serializer=self.json)

    def test_Gets_basic_info_from_nibabel_and_returns_it(self):
        out = self.file.inspect()
        self.assertEqual(out['subject'], 'John Doeish')
        self.assertEqual(out['protocol'], 'T1 SENSE')
        self.assertEqual(out['acquired'], datetime(2014, 8, 5, 11, 27, 34))

    def test_Gets_dimensions(self):
        out = self.file.inspect()
        self.assertEqual(out['dimensions'], [80,80,10])

    def setupNibabel(self):
        img = Mock()
        img.header.general_info = {
            'exam_date':'2014.08.05 / 11:27:34',
            'protocol_name':'T1 SENSE',
            'patient_name':'John Doeish'}
        img.shape = (80,80,10)
        self.libs.nibabel.load.return_value = img
        self.libs.hasDependency.return_value = True




import unittest
from mock import Mock
import os


class FileFilterTests(unittest.TestCase):

    def test_reads_extensions_from_default_file(self):
        import niprov.filefilter
        filesys = Mock()
        filesys.readlines.return_value = ['.abc','.def']
        niprov.filefilter.pkg_resources = Mock()
        filt = niprov.filefilter.FileFilter(filesys=filesys)
        niprov.filefilter.pkg_resources.resource_filename.assert_called_with(
            'niprov','discovery-filter.txt')
        filesys.readlines.assert_called_with(
            niprov.filefilter.pkg_resources.resource_filename())
        self.assertTrue(filt.include('/p/sth.abc'))
        self.assertTrue(filt.include('/p/sth.def'))
        self.assertFalse(filt.include('/p/sth.xyz'))





import unittest
from mock import Mock

class ReportingTests(unittest.TestCase):

    def setUp(self):
        self.factory = Mock()
        self.repo = Mock()
        self.listener = Mock()
        self.exporter = Mock()
        self.factory.createExporter.return_value = self.exporter

    def report(self, *args, **kwargs):
        import niprov
        return niprov.report(*args, repository=self.repo, exportFactory=self.factory,
            listener=self.listener, **kwargs)

    def test_Without_specifics_returns_all_files(self):
        out = self.report()
        self.exporter.export.assert_called_with(self.repo.all())
        self.assertEqual(out, self.exporter.export())

    def test_Can_report_on_one_file_specifically(self):
        out = self.report(forFile='afile.f')
        self.exporter.export.assert_called_with(self.repo.byPath('afile.f'))
        self.assertEqual(out, self.exporter.export())

    def test_Can_report_on_all_files_for_subject(self):
        out = self.report(forSubject='Jane Doe')
        self.exporter.export.assert_called_with(
            self.repo.bySubject('Jane Doe'))
        self.assertEqual(out, self.exporter.export())

    def test_Obtains_exporter_from_factory(self):
        import niprov
        factory = Mock()
        repo = Mock()
        niprov.report(repository=repo, exportFactory=factory)
        factory.createExporter.assert_any_call(None, None)
        niprov.report(medium='html', repository=repo, exportFactory=factory)
        factory.createExporter.assert_any_call('html', None)
        niprov.report(form='narrative', repository=repo, exportFactory=factory)
        factory.createExporter.assert_any_call(None, 'narrative')

    def test_Passes_provenance_to_exporter(self):
        import niprov
        factory = Mock()
        repo = Mock()
        niprov.report(repository=repo, exportFactory=factory)
        factory.createExporter().export.assert_any_call(repo.all())
        niprov.report(forSubject='Jane', repository=repo, exportFactory=factory)
        factory.createExporter().export.assert_any_call(repo.bySubject())

    def test_Passes_single_item_of_provenance_to_exporter(self):
        import niprov
        factory = Mock()
        repo = Mock()
        niprov.report(forFile='xyz',repository=repo, exportFactory=factory)
        factory.createExporter().export.assert_any_call(repo.byPath('xyz'))

    def test_If_file_unknown_tells_listener(self):
        self.repo.knowsByPath.return_value = False
        self.repo.byPath.side_effect = IndexError
        self.report(forFile='xyz')
        self.listener.unknownFile.assert_called_with('xyz')






import unittest
from mock import Mock
import os.path as ospath

class FilecomparisonTests(unittest.TestCase):


    def test_Hasher_provides_unique_hashes(self):
        import niprov.hashing
        hasher = niprov.hashing.Hasher()
        file1 = __file__
        file2 = niprov.hashing.__file__
        self.assertEqual(hasher.digest(file1), hasher.digest(file1))
        self.assertNotEqual(hasher.digest(file1), hasher.digest(file2))
